COMMENT  ^

Contains the register definitions for the RTL8029 Network Interface Controller.
For register definitions, see the data sheet for the RTL8029.

^
;==============================================================================
;Register address assignments
;==============================================================================
IDR0    EQU     00h     ;ID Register 0  (R/W)
IDR1    EQU     01h     ;ID Register 1  (R/W)
IDR2    EQU     02h     ;ID Register 2  (R/W)
IDR3    EQU     03h     ;ID Register 3  (R/W)
IDR4    EQU     04h     ;ID Register 4  (R/W)
IDR5    EQU     05h     ;ID Register 5  (R/W)
               ;06h     reserved
               ;07h     reserved
MAR0    EQU     08h     ;Multicast Register 0  (R/W)
MAR1    EQU     09h     ;Multicast Register 1  (R/W)
MAR2    EQU     0Ah     ;Multicast Register 2  (R/W)
MAR3    EQU     0Bh     ;Multicast Register 3  (R/W)
MAR4    EQU     0Ch     ;Multicast Register 4  (R/W)
MAR5    EQU     0Dh     ;Multicast Register 5  (R/W)
MAR6    EQU     0Eh     ;Multicast Register 6  (R/W)
MAR7    EQU     0Fh     ;Multicast Register 7  (R/W)
TSD0    EQU     10h     ;Transmit Status of Descriptor 0 (R/W)
TSD1    EQU     14h     ;Transmit Status of Descriptor 1 (R/W)
TSD2    EQU     18h     ;Transmit Status of Descriptor 2 (R/W)
TSD3    EQU     1Ch     ;Transmit Status of Descriptor 3 (R/W)
TSAD0   EQU     20h     ;Transmit Start Address of Descriptor 0 (R/W)
TSAD1   EQU     24h     ;Transmit Start Address of Descriptor 1 (R/W)
TSAD2   EQU     28h     ;Transmit Start Address of Descriptor 2 (R/W)
TSAD3   EQU     2Ch     ;Transmit Start Address of Descriptor 3 (R/W)
RBSTART EQU     30h     ;Receive Buffer Start Address (R/W)
ERBCR   EQU     34h
ERSR    EQU     36h
CMD     EQU     37h     ;Command Register (R/W)
CAPR    EQU     38h     ;Current Address of Packet Read (R/W)
IMR     EQU     3Ch     ;Interrupt Mask Register (R/W)
ISR     EQU     3Eh     ;Interrupt Status Register (R/W)
TCR     EQU     40h     ;Transmit Configuration Register (R/W)
RxCR    EQU     44h     ;Receive Configuration Register (R/W)
TCTR    EQU     48h     ;Timer CounT Register (R/W)
MPC     EQU     4Ch     ;Missed Packet Counter (R)
CR9346  EQU     50h     ;9346 Command Register (R/W)
CONFIG0 EQU     51h     ;Config 0 (R/W)
CONFIG1 EQU     52h     ;Config 1 (R/W)
Flash   EQU     54h     ;Flash Memory read/write Register (R/W)
GEP     EQU     58h     ;General purpose Register (R/W)
GEPCTL  EQU     59h     ;General purpose Control Register (R/W)
MIIR    EQU     5Ah     ;MII Register (R/W)
HLTCLK  EQU     5Bh     ;Halt Clock Register (W)
MULINT  EQU     5Ch     ;Multiple Interrupt (R/W)
TSAD    EQU     60h     ;Transmit Status of all Descriptors (R)
MII0    EQU     62h     ;MII0 register
FIFOTMS EQU     70h     ;70h - 73h FIFO test mode select
FIFOTMD EQU     74h     ;74h - 77h FIFO test mode select
CLRCACHE EQU    7Fh     ;7Fh clear cache (PCS mode)
;; DOS ODI 8129
;----------------------------------------------------------------------------
;       8129 ISR(IMR) bits
;----------------------------------------------------------------------------
ISR_ROK                 equ     01h            ;receive OK
ISR_RER                 equ     02h   ;receive error, collided frame (<16 byte)
                                      ; will not set this bit
ISR_TOK                 equ     04h   ;transmit OK
ISR_TER                 equ     08h   ;transmit error
ISR_RXOVW               equ     10h   ;rx buffer overflow
ISR_PUN                 equ     20h   ;rx packet underrun (CAPR over-write)
ISR_SERR                equ     80h   ;PCI system error
;----------------------------------------------------------------------------

;----------------------------------------------------------------------------
;       8129 TCR bits
;----------------------------------------------------------------------------
TCR_CLRABT              equ    0001h             ;clear abort (re-transmit)
TCR_SNPAC               equ    0002h             ;send next packet(re-tx next)
TCR_CRC                 equ    00010000h         ;1: CRC not appended
TCR_LBK0                equ    00020000h         ;loopback test 0
TCR_LBK1                equ    00040000h         ;loopback test 1
TCR_IFG0                equ    01000000h         ;Interframe Gap0
TCR_IFG1                equ    02000000h         ;Interframe Gap1
;TxBurstDMA              dw     0100h             ;Max DMA burst
                                        ;0000 - 16  byte
                                        ;0100 - 32  byte
                                        ;0200 - 64  byte
                                        ;0300 - 128 byte
                                        ;0400 - 256 byte
                                        ;0500 - 512 byte
                                        ;0600 -1024 byte
                                        ;0700 - unlimted
Burst16                 equ     0000h
Burst32                 equ     0100h
Burst64                 equ     0200h
Burst128                equ     0300h
Burst256                equ     0400h
Burst512                equ     0500h
Burst1024               equ     0600h
BurstUnlimited          equ     0700h
;----------------------------------------------------------------------------


;----------------------------------------------------------------------------
;       8129 RCR bits
;----------------------------------------------------------------------------
RCR_AAP                 equ    0001h             ;accept all physical
RCR_APM                 equ    0002h             ;accept physical match
RCR_AM                  equ    0004h             ;accept multicast
RCR_AB                  equ    0008h             ;accept broadcast
RCR_AR                  equ    0010h             ;accept runt packet
RCR_AER                 equ    0020h             ;accept error packet
RCR_RBLEN0              equ    0800h             ;rx buffer length0
RCR_RBLEN1              equ    1000h             ;00=8k, 01=16k, 10=32k, 11=64k
;RxBurstDMA              dw     0100h             ;Max DMA burst
                                        ;bit 10-8 (000 - 111)
                                        ;0000 - 16  byte
                                        ;0100 - 32  byte
                                        ;0200 - 64  byte
                                        ;0300 - 128 byte
                                        ;0400 - 256 byte
                                        ;0500 - 512 byte
                                        ;0600 -1024 byte
                                        ;0700 - unlimted

RESV4	equ	00004000h
RESV5	equ	0000E000h

;----------------------------------------------------------------------------
;       8129 (RSR) Rx Status bits  (rx header)
;----------------------------------------------------------------------------
RSR_ROK                 equ    0001h             ;Received Packet intact
RSR_FAE                 equ    0002h             ;frame aligned error
RSR_CRC                 equ    0004h             ;CRC error
RSR_LONG                equ    0008h             ;long packet > 8k byte
RSR_RUNT                equ    0010h             ;runt packet < 64 byte
RSR_ISE                 equ    0020h             ;invalid symbol
RSR_BAR                 equ    2000h            ;broadcast packet received
RSR_PAM                 equ    4000h            ;physical packet matched received
RSR_MAR                 equ    8000h            ;multicast packet received
;----------------------------------------------------------------------------

;----------------------------------------------------------------------------
;       8129 (TSAD bits) Tx Status of All Descriptors
;----------------------------------------------------------------------------
TSAD_OWN0               equ    0001h             ;OWN bit of descriptor 0
TSAD_OWN1               equ    0002h             ;OWN bit of descriptor 1
TSAD_OWN2               equ    0003h             ;OWN bit of descriptor 2
TSAD_OWN3               equ    0008h             ;OWN bit of descriptor 3
TSAD_TABT0              equ    0010h             ;TABT bit of descriptor 0
TSAD_TABT1              equ    0020h             ;TABT bit of descriptor 1
TSAD_TABT2              equ    0040h             ;TABT bit of descriptor 2
TSAD_TABT3              equ    0080h             ;TABT bit of descriptor 3
TSAD_TUN0               equ    0100h             ;TUN bit of descriptor 0
TSAD_TUN1               equ    0200h             ;TUN bit of descriptor 1
TSAD_TUN2               equ    0400h             ;TUN bit of descriptor 2
TSAD_TUN3               equ    0800h             ;TUN bit of descriptor 3
TSAD_TOK0               equ    1000h             ;TOK bit of descriptor 0
TSAD_TOK1               equ    2000h             ;TOK bit of descriptor 1
TSAD_TOK2               equ    4000h             ;TOK bit of descriptor 2
TSAD_TOK3               equ    8000h             ;TOK bit of descriptor 3
;----------------------------------------------------------------------------

;----------------------------------------------------------------------------
;       8129 (CMD) command register bits
;----------------------------------------------------------------------------
CMD_RST                 equ     10h
CMD_RE                  equ     08h
CMD_TE                  equ     04h
CMD_EMPTY               equ     01h
;----------------------------------------------------------------------------

;----------------------------------------------------------------------------
;       8129 (ERSR)
;----------------------------------------------------------------------------
EROK                    equ     1
EROVW                   equ     2
ERBad                   equ     4
ERGood                  equ     8
;----------------------------------------------------------------------------

;----------------------------------------------------------------------------
;       8129 (CR9346) 9346 command register bits
;----------------------------------------------------------------------------
CR9346_EEDO             equ    01h              ;9346 data out
CR9346_EEDI             equ    02h              ;9346 data in
CR9346_EESK             equ    04h              ;9346 serial clock
CR9346_EECS             equ    08h              ;9346 chip select
CR9346_EEM0             equ    40h              ;select 8129 operating mode
CR9346_EEM1             equ    80h              ;00: normal
                                                ;01: autoload
                                                ;10: 9346 programming
                                                ;11: config write enable
;----------------------------------------------------------------------------

;----------------------------------------------------------------------------
;       8129 (CONFIG0) CONFIG0 register bits
;----------------------------------------------------------------------------
CONFIG0_PL0             equ    08h              ;10Mps PL0
CONFIG0_PL1             equ    10h              ;10Mps PL1
                ;PL1, PL0   00  auto-detect (100PCS, 10BaseT, 10Base2)
                ;           01  force 10BaseT with link test failed
                ;           10  force 10Base5
                ;           11  force 10Base2
CONFIG0_T10             equ    20h              ;10Mps select
CONFIG0_PCS             equ    40h              ;100 PCS select
CONFIG0_SCR             equ    80h              ;scramble/descramble select
;----------------------------------------------------------------------------

;----------------------------------------------------------------------------
;       8129 (CONFIG1) CONFIG1 register bits
;----------------------------------------------------------------------------
CONFIG1_PWRDN           equ    01h              ;power down mode
CONFIG1_SLEEP           equ    02h              ;sleep mode
CONFIG1_IOMAP           equ    04h              ;I/O mapping
CONFIG1_MEMMAP          equ    08h              ;memory mapping
CONFIG1_LEDSEL          equ    10h              ;0: LEDCRS,  1: LEDTX
                                                ; only valid in PCS mode
CONFIG1_DVRLOAD         equ    20h
                ;It will be cleared when writing to PCI's IOEN, MEMEN, BMEN
CONFIG1_FUDUP           equ    40h              ;full duplex selected
CONFIG1_PCIWAIT         equ    80h              ;insert wait state at IDRY
;----------------------------------------------------------------------------

;------------------------------------------------------------------------------
;                       STRUCTURE NAME --- PKT_STORAGE
;------------------------------------------------------------------------------
;Packet storage format

 PKT_STORAGE     STRUC
 pkt_status      dw      ?      ; packet status
 pkt_count       dw      ?      ; number of bytes in packet
 pkt_data        db      ?      ; start of packet data
 PKT_STORAGE     ENDS

