COMMENT ^

  This include file provides some common definitions and various
  abstractions for segment names, procedure names, etc., for the
  microsoft assembler (MASM) version 5.0.

- Segment names:

  Following macros abstract the segment declarations.  In the program
  source the following macros should be used rather than the full
  segment declarations.  Note the purpose of these macros is to keep
  the program source independent of segment names and placement of
  procedures and variables in certain segments, thus facilitating
  independence from the compiler and the model.

  Defaults defined in this file are indicated in the descriptions.
  These default can be overridden by defining the desired values
  in the MASM command line.

  DATASEG_BEG   defines the beginning of data segment.
  DATASEG_END   defines the end of data segment

  CODESEG_BEG   defines the beginning of the program segment.  The ASSUME
                statement for the CS and DS is also defined, using
                CGRPNAM and DGRPNAM, respectively.  However, for ES
                segment register, the ASSUME must be explicitly stated.
  CODESEG_END   defines the end of the code segment


- Assembly procedures

  These macros allow definitions of procedures to be called by
  other assembly routines.
  The default type for A_PROC is NEAR, and can be overriden by
  explicitly specifying the type.

  A_PROC <name> [<type>]        a public procedure
  A_ENDP <name>                 end of a public procedure

  L_PROC <name>                 a local procedure
  L_ENDP <name>                 end of local procedure


^
;
;       define some commonly used, simple values
;

        NONE    EQU     -1      ; general nil value
        NULL    EQU      0      ; C NULL pointer
        TRUE    EQU      1      ; logical true to conform to C convention
        FALSE   EQU      0      ; logical false

;
;       from now on, do stuff which is only necessary and useful during
;       the first pass.
;
IF1                             ; on Pass 1 only


;
; public assembly routine definitions
;
A_PROC  MACRO   PN,PT
        PUBLIC  &PN
        IFB     <PT>
                &PN     PROC    NEAR
        ELSE
                &PN     PROC    &PT
        ENDIF
ENDM

A_ENDP  MACRO   PN
        &PN     ENDP
ENDM

;
; local assembly routine definitions
;
L_PROC  MACRO   PN
IF DEBUG
        PUBLIC  &PN
ENDIF
        &PN     PROC    NEAR
ENDM

L_ENDP  MACRO   PN
        &PN     ENDP
ENDM

ENDIF   ; end of IF1, stuff assembled only on first pass


DATASEG_BEG     MACRO
IFDEF DOS
        _DATA SEGMENT DWORD PUBLIC 'CODE'
ELSE
        _DATA SEGMENT DWORD PUBLIC 'DATA'
ENDIF
        ENDM

DATASEG_END     MACRO
        _DATA ENDS
        ENDM


CODESEG_BEG     MACRO
        _TEXT SEGMENT DWORD PUBLIC 'CODE'
IFDEF   DOS
        ASSUME  CS:DGROUP
        ASSUME  DS:DGROUP
        .386P
ELSE
        .386P
        ASSUME  CS:CGROUP
        ASSUME  DS:DGROUP
ENDIF
ENDM
        .286P
CODESEG_END     MACRO
        _TEXT ENDS
ENDM



INITDATASEG_BEG MACRO
IFDEF DOS
        _INITDATA SEGMENT DWORD PUBLIC 'CODE'
ELSE
        _INITDATA SEGMENT DWORD PUBLIC 'DATA'
ENDIF
        ENDM

INITDATASEG_END MACRO
        _INITDATA ENDS
        ENDM


INITCODESEG_BEG MACRO
        _INITTEXT SEGMENT DWORD PUBLIC 'CODE'
IFDEF   DOS
        ASSUME  CS:DGROUP
        ASSUME  DS:DGROUP
        .386P
ELSE
        ASSUME  CS:CGROUP
        ASSUME  DS:DGROUP
        .386P
ENDIF
ENDM
        .286P

INITCODESEG_END MACRO
        _INITTEXT ENDS
ENDM




;Group Definitions and segment order

IFDEF DOS

_DATA SEGMENT DWORD PUBLIC 'CODE'
_DATA ENDS
_TEXT SEGMENT DWORD PUBLIC 'CODE'
_TEXT ENDS
_INITDATA SEGMENT DWORD PUBLIC 'CODE'
_INITDATA ENDS
_INITTEXT SEGMENT DWORD PUBLIC 'CODE'
_INITTEXT ENDS
DGROUP GROUP _DATA, _TEXT, _INITDATA, _INITTEXT

ELSE

_DATA SEGMENT DWORD PUBLIC 'DATA'
_DATA ENDS
_TEXT SEGMENT DWORD PUBLIC 'CODE'
_TEXT ENDS
_INITDATA SEGMENT DWORD PUBLIC 'DATA'
_INITDATA ENDS
_INITTEXT SEGMENT DWORD PUBLIC 'CODE'
_INITTEXT ENDS

DGROUP GROUP _DATA, _INITDATA
CGROUP GROUP _TEXT, _INITTEXT

ENDIF


