COMMENT  ^

Contains the data structures and constants used in the driver.
^
IFNDEF MACSRV_BCAST         ; if not define MACSRV_BCAST
include vector.inc
ENDIF

;------------------------------------------------------------------------------
;Version constant
;------------------------------------------------------------------------------
MAJORVERS       equ     0   ;Version 1.0
MINORVERS       equ     0
;------------------------------------------------------------------------------
;Service flags : broadcast, multicast, promisicuous, settable station address,
;                GDT virtual address.
;------------------------------------------------------------------------------
SERVFL          equ     MACSRV_BCAST + MACSRV_MCAST + MACSRV_PROMIS \
                         + MACSRV_SETSNPA +  MACSRV_GDTADDR +  MACSRV_INTR
;------------------------------------------------------------------------------
;Module name : RTL8129
;------------------------------------------------------------------------------
;module name
MODULENAME      equ     "NETWORK_NIF"

;------------------------------------------------------------------------------
;Driver wide constants
;------------------------------------------------------------------------------
 BUFSIZE         equ     1536D;RxBufferSize   ;size of buffers ?
;------------------------------------------------------------------------------
;CRC not included in length constants
;------------------------------------------------------------------------------
MINPACKET       equ     60d             ;minimum packet size
MAXPACKET       equ     1514d           ;maximum packet size
;------------------------------------------------------------------------------
;Number of multicast slots
;------------------------------------------------------------------------------
MCASTSZ         equ     5               ;number of multicast addresses

;----------------------------------------------------------------------------
;       8129 MIIR bits
;----------------------------------------------------------------------------
MIIR_MDM        equ     80h
MIIR_MDIOL      equ     00h
MIIR_MDIOH      equ     04h
MIIR_MDCH       equ     01h
MIIR_MDCL       equ     00h


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;; utility ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;------------------------------------------------------------------------------
;                          MACRO NAME --- DELAY
;------------------------------------------------------------------------------
DELAY           MACRO   countNo
                PUSH    CX
                MOV     CX, countNo
@@:             JMP     $+2
                LOOP    @B
                POP     CX
                ENDM
;------------------------------------------------------------------------------
;                          MACRO NAME --- DWADD
;------------------------------------------------------------------------------
;macro for adding to double words (used for statistics)
DWADD           MACRO   addr,num
                add     WORD PTR addr,num
                adc     WORD PTR addr[2],0
                ENDM
;------------------------------------------------------------------------------
;                    MACRO NAME --- pushc
;------------------------------------------------------------------------------
;Macro for pushing constants
pushc           MACRO   immed
IFDEF           DOS                     ;DOS
                mov     ax,immed
                push    ax
ELSE                                    ;OS/2
                push    immed
ENDIF
                ENDM
;------------------------------------------------------------------------------
; PIC address
;------------------------------------------------------------------------------
PIC1COM         EQU     20H             ;register for EOI on PIC 1
PIC1MASK        EQU     21H             ;register for masking on PIC 1
PIC2COM         EQU     0A0H            ;register for EOI on PIC 2
PIC2MASK        EQU     0A1H            ;register for masking on PIC 2
EOI             EQU     20H             ;non-specific EOI
;------------------------------------------------------------------------------
;                         MACRO NAME --- PRINT


Slowly  macro   Number
        local   lp_slow
        push    ax
        push    cx
        mov     cx, Number
lp_slow:
        in      al, 61h
        loop    lp_slow
        pop     cx
        pop     ax
        endm

;------------------------------------------------------------------------------
;DOS debugging macro (output a string on screen)
PRINT   MACRO   arg
IFDEF   DOS
        push    ax              ;reserve the values of ax, dx
        push    dx
        mov     dl,arg          ;put output value into DL reg.
        mov     ah,2
        int     21h             ;output a character
        pop     dx              ;restore ax, dx
        pop     ax
ENDIF
        ENDM
;------------------------------------------------------------------------------
;                          MACRO NAME --- PULSE
;------------------------------------------------------------------------------
;performance measurement macro used with oscilloscope.
PULSE   MACRO
        push    dx
        push    ax
        mov     dx,320h
        in      ax,dx
        pop     ax
        pop     dx
        ENDM

; <<Set for RTL8129 memory organization>>
OneSecond       EQU     33333333d

TxBufferSize    EQU     1600d         ;pages in transmit buffer
TxBufferNo      EQU     8             ;# of transmit buffers


RxBufferSize    EQU     1024*16

PROMAddr        EQU     0000H        ;ethernet address PROM


;---------------------------------------------------------------------------
;Interrupts we care about
;---------------------------------------------------------------------------
;MYINTS          EQU     ISR_PRX+ISR_PTX+ISR_TXE+ISR_OVW+ISR_CNT

MYINTS      EQU     802Fh ;ISR_ROK+ISR_RER+ISR_TOK+ISR_TER+ISR_RXOVW+ISR_SERR


;*****************************************************************************
;VDS Service........VDS Service........VDS Service........VDS Service........
; Used for supporting HighMem
;*****************************************************************************
;VDS interrupt number
        VDS_INTERRUPT       EQU       4Bh

; DMA Services function code
        VDS_GET_VERSION     EQU     8102h
        VDS_LOCK_REGION     EQU     8103h
        VDS_UNLOCK_REGION   EQU     8104h
        VDS_SC_GA_LOCK      EQU     8105h   ; Scatter/Gather lock region
        VDS_SC_GA_UNLOCK    EQU     8106h   ; Scatter/Gather unlock region
        VDS_REQUEST_BUFFER  EQU     8107h   ; Request DMA Buffer
        VDS_RELEASE_BUFFER  EQU     8108h   ; Release DMA Buffer
        VDS_COPY_IN_BUFFER  EQU     8109h
        VDS_COPY_OUT_BUFFER EQU     810Ah
        VDS_DISABLE_TRANSL  EQU     810Bh
        VDS_ENABLE_TRANSL   EQU     810Ch

; Structure required for passing parameters to VDS services.

;---------------------------------------------------------------------------
;                   MACRO NAME --- WRD_NIC WTITE A Double Word
;---------------------------------------------------------------------------
WRD_NIC MACRO   REGS, VALUE     ; write VALUE into RTL8129 register
        PUSH    EAX
        PUSH    EDX
        MOV     DX, CS:IOBase
        ADD     DX, REGS
        MOV     EAX, VALUE
        OUT     DX, EAX
        JMP     $+2
        POP     EDX
        POP     EAX
        ENDM

;---------------------------------------------------------------------------
;                   MACRO NAME --- WR_NIC Write a Word
;---------------------------------------------------------------------------
WR_NIC  MACRO   REGS, VALUE     ; write VALUE into RTL8129 register
        PUSH    AX
        PUSH    DX
        MOV     DX, CS:IOBase
        ADD     DX, REGS
        MOV     AX, VALUE
        OUT     DX, AX
        JMP     $+2
        POP     DX
        POP     AX
        ENDM

;---------------------------------------------------------------------------
;               MACRO NAME --- RD_NIC  Read a Word
;---------------------------------------------------------------------------
RD_NIC  MACRO   VALUE, REGS
        PUSH    AX
        PUSH    DX
        MOV     DX, CS:IOBase
        ADD     DX, REGS        ;DX = IOBase+REGS
        IN      AX, DX
        JMP     $+2
        MOV     VALUE, AX
        POP     DX
        POP     AX
        ENDM

;---------------------------------------------------------------------------
;            MACRO NAME --- RDD_NIC Read a Double Word
;---------------------------------------------------------------------------
RDD_NIC MACRO   VALUE, REGS
        PUSH    EAX
        PUSH    DX
        MOV     DX, CS:IOBase
        ADD     DX, REGS        ;DX = IOBase+REGS
        IN      EAX, DX
        JMP     $+2
        MOV     VALUE, EAX
        POP     DX
        POP     EAX
        ENDM

DOWN    MACRO
        push    es
        push    ax
        mov     ax,0b800h
        mov     es,ax
        mov     byte ptr es:0140d,044h
        mov     byte ptr es:0142d,04fh
        mov     byte ptr es:0144d,057h
        mov     byte ptr es:0146d,04eh
        pop     ax
        pop     es
        ENDM

T100    MACRO
        push    es
        push    ax
        mov     ax,0b800h
        mov     es,ax
        mov     byte ptr es:0140d,031h
        mov     byte ptr es:0142d,030h
        mov     byte ptr es:0144d,030h
        mov     byte ptr es:0146d,04dh
        pop     ax
        pop     es

        ENDM

T10     MACRO
        push    es
        push    ax
        mov     ax,0b800h
        mov     es,ax

        mov     byte ptr es:0140d,031h
        mov     byte ptr es:0142d,030h
        mov     byte ptr es:0144d,04dh
        mov     byte ptr es:0146d,020h

        pop     ax
        pop     es
        ENDM


MIIRSetLow      macro
        mov     al, MIIR_MDM+MIIR_MDIOL+MIIR_MDCL
        out     dx, al
        ;slowly  2
        jmp     $+2                                     ;           ---
        jmp     $+2                                                     ;          |   |
        mov     al, MIIR_MDM+MIIR_MDIOL+MIIR_MDCH       ;MDC     --|   |--
        out     dx, al                                  ;
        ;slowly  2                                      ;     --
        ;jmp     $+2                                     ;MDIO \ |
                                                        ;     --|---------
        mov     al, MIIR_MDM+MIIR_MDIOL+MIIR_MDCL
        out     dx, al

        endm

MIIRSetHigh     macro
        mov     al, MIIR_MDM+MIIR_MDIOH+MIIR_MDCL
        out     dx, al
        ;slowly  2
        jmp     $+2
        jmp     $+2
        jmp     $+2

        mov     al, MIIR_MDM+MIIR_MDIOH+MIIR_MDCH      ;           ---
        out     dx, al                                 ;          |   |
        ;slowly  2                                     ;MDC     --|   |--
        ;jmp     $+2                                    ;
                                                       ;     -- ---------
        mov     al, MIIR_MDM+MIIR_MDIOH+MIIR_MDCL      ;MDIO \ |
        out     dx, al                                 ;     --|
        endm

MIIRInput       macro
        mov     al, MIIR_MDCL
        out     dx, al
        ;;slowly        1
        jmp     $+2
        jmp      $+2
        jmp      $+2
        jmp     $+2
        in      al, dx

        push    ax

        mov     al, MIIR_MDCH
        out     dx, al
        ;jmp     $+2
        ;jmp     $+2
        ;jmp     $+2

        mov     al, MIIR_MDCL
        out     dx, al

        pop     ax
        endm


