COMMENT ^
This is the header file for Protocol Manager and its clients.  It
contains request block structure for the Protocol Manager Primitives
(Request to the protocol manager), as well as error codes and opcode
definitions.

Note that error codes are in singluar domain.  And opcodes are in three
different domains; PM primitives, Module System requests, and general
MAC requests.

This header file is based on Microsoft's
"Lan Manager Network Driver Interface Specification", Version 1.0
^
;==============================================================================
; VECTOR SYSTEM RETURN CODES
; =============================================================================
; Reference : LAN Manager Network Driver Interface Spec, Ver 1.0
;
; NOTE : 0xF000 thru 0xFFFF are vendor-defined error returns,
; are to be treated as GENERAL_FAILURE

SUCCESS 		EQU	0000h
WAIT_FOR_RELEASE	EQU	0001h
REQUEST_QUEUED		EQU	0002h
FRAME_NOT_RECOGNIZED	EQU	0003h
FRAME_REJECTED		EQU	0004h
FORWARD_FRAME		EQU	0005h
OUT_OF_RESOURCE 	EQU	0006h
INVALID_PARAMETER	EQU	0007h
INVALID_FUNCTION	EQU	0008h
NOT_SUPPORTED		EQU	0009h
HARDWARE_ERROR		EQU	000Ah
ALREADY_STARTED 	EQU	0020h
INCOMPLETE_BINDING	EQU	0021h
DRIVER_NOT_INITIALIZED	EQU	0022h
HARDWARE_NOT_FOUND	EQU	0023h
HARDWARE_FAILURE	EQU	0024h
CONFIGURATION_FAILURE	EQU	0025h
INTERRUPT_CONFLICT	EQU	0026h
INCOMPATIBLE_MAC	EQU	0027h
INITIALIZATION_FAILED	EQU	0028h
GENERAL_FAILURE 	EQU	00FFh
;==============================================================================
; The following functions have direct entry points and
; do not require function codes :
;
; MAC's UPPER-DISPATCH-TABLE (UDT) : support protocol to call
;	a) Request		b) Transmit_Chain
;	c) Transfer_Data	d) Receive_Release
;	e) Indication_On	f) Indication_Off
;
; PROTO's LOWER-DISPATCH-TABLE (LDT) : support MAC to call
;	a) Request_Confirm	b) Transmit_Confirm
;	c) Receive_Lookahead	d) Indication_Complete
;	f) Receive_Chain	g) Status_Indication
; =============================================================================
; REQUESTS - FUNCTION CODES : IN UDT
;------------------------------------------------------------------------------
; The following commands are issued to MAC-driver, using its Request
; entry point.

REQ_RUN_DIAGNOSTICS	EQU	0001h	; initiate diagnostics
REQ_GET_ERROR_LOG	EQU	0002h	; read error log
REQ_SET_LOCSNPA 	EQU	0003h	; set station address
REQ_OPEN_DEV		EQU	0004h	; open adapter
REQ_CLOSE_DEV		EQU	0005h	; close adapter
REQ_RESET_DEV		EQU	0006h	; reset MAC
REQ_SET_PKTFLT		EQU	0007h	; set packet filter
REQ_SET_MULTICAST	EQU	0008h	; add multicast address
REQ_DEL_MULTICAST	EQU	0009h	; delete multicast address
REQ_UPDATE_STATS	EQU	000Ah	; update statistics
REQ_CLR_STATS		EQU	000Bh	; clear statistics
REQ_INTERRUPT		EQU	000Ch	; interrupt
REQ_SET_FUNCADDR	EQU	000Dh	; set functional address
REQ_SET_LOOKAHEAD	EQU	000Eh	; set lookahead
;------------------------------------------------------------------------------
; STATUS INDICATIONS - FUNCTION CODES :
;------------------------------------------------------------------------------
; The following commands are asynchronous indications from
; MAC to provide status information. MAC calls PROTO's
; status_indiciation() handler (UDT).

MAC_RING_STATUS 	EQU	1	; ring status
MAC_DEV_CHECK		EQU	2	; adpater check
MAC_START_RESET 	EQU	3	; start reset
;------------------------------------------------------------------------------
; SYSTEM REQUEST (PROTOCOL-MANAGER RELEATED) - FUNCTION CODES :
;------------------------------------------------------------------------------
; The following commands are synchronous system requests
; between the Protocol Manager <--> Module, during
; system starup time for configuration and binding.

VEC_INITIATE_BIND	EQU	1	; Initiate Bind
VEC_BIND		EQU	2	; Bind
;------------------------------------------------------------------------------
; PROTOCOL MANAGER PRIMITIVES
;------------------------------------------------------------------------------
; In OS2 :	Protocol/MAC modules issue OS2 IOCTL to Protocol-Manager
;		device driver to get configuration information etc.
; In DOS :	Protocol/MAC modules issue DOS IOCTL to Protocol_Manager
;		device driver to get configuration information etc.

PM_GET_PM_INFO		EQU	1	; Get Protocol Manager Info
PM_REG_MODULE		EQU	2	; Register Module
PM_BIND_START		EQU	3	; Bind and Start
PM_GET_PM_LINKAGE	EQU	4	; Get Protocol Manager Linkage
;------------------------------------------------------------------------------
;		   STRUCTURE NAME --- pm_ioctl_reqblk
;------------------------------------------------------------------------------
; OS2/DOS IOCTL Request Block structure, used in IOCTL
; transcations with the Protocol Manger.

pm_ioctl_reqblk 	STRUC
io_opcode	dw	?		; Opcode for Protocol MAnager request
io_status	dw	?		; Status at completion of request
io_pointer1	dd	?		; First parameter Ring 0 GDT pointer
io_pointer2	dd	?		; Second parameter Ring 0 GDT pointer
io_word1	dw	?		; parameter word 1
pm_ioctl_reqblk 	ENDS
;------------------------------------------------------------------------------
; FUNCTION-FLAGS : cc_func_flags (ConnChar)
FUNC_FLAG_UPPER_BIND	EQU	1	; upper boundary binding supported
FUNC_FLAG_LOWER_BIND	EQU	2	; lower boundary binding supported

;------------------------------------------------------------------------------
; protocol-level at module's boundary (CommChar)
PHYS_PROTO_LEVEL	EQU	0	; physical level
MAC_PROTO_LEVEL 	EQU	1	; mac-level
LINK_PROTO_LEVEL	EQU	2	; datalink-level
NET_PROTO_LEVEL 	EQU	3	; network-level
XPORT_PROTO_LEVEL	EQU	4	; transport-level
SESS_PROTO_LEVEL	EQU	5	; session-level
UNSPEC_PROTO_LEVEL	EQU	-1	; unspecifed level

;------------------------------------------------------------------------------
; interface-type at a given protocol-level (CommChar)
MAC_MAC_INTF		EQU	1	; At MAC-LEVEL : mac interface type
SESS_NETBIOS_INTF	EQU	1	; At SESS-LEVEL : netbios interface
;------------------------------------------------------------------------------
; FAR PASCAL FUNCTION POINTER (FP_FUNCPT)

; MODULE NAME
VEC_MODNAME_SZ		EQU	16d
; MAC SERVICE TYPE
MAC_SRVTYPE_SZ		EQU	16d
; MAC SNPA address
MAC_SNPA_SZ		EQU	16d
;------------------------------------------------------------------------------
;		       STRUCTURE NAME --- mcast_tbl
;------------------------------------------------------------------------------
; MULTICAST ADDRESS TABLE
mcast_tbl	STRUC
mt_mxentry	dw	?	; max number of entry count
mt_curentry	dw	?	; current entry count
mt_mcast	db	MAC_SNPA_SZ dup (?)	; first MCAST address entry
mcast_tbl	ENDS
;------------------------------------------------------------------------------
; MAC SERVICE FLAGS (mc_srvflags) (ServChar)
MACSRV_BCAST	EQU	1h	; Mac-driver supports broadcast
MACSRV_MCAST	EQU	2h	; Mac-driver supports multicast
MACSRV_FUNC	EQU	4h	; Mac-driver supports functional addr
MACSRV_PROMIS	EQU	8h	; Mac-driver supports promiscuous mode
MACSRV_SETSNPA	EQU	10h	; Mac-driver supports set-SNPA-address maccmd
MACSRV_STATS	EQU	20h	; Mac-driver stats always current
MACSRV_DIAG	EQU	40h	; Mac-driver supports Initiate_Diagnostics
MACSRV_LOOPBACK EQU	80h	; Mac-driver level Loopback supported
MACSRV_RCVTYPE	EQU	100h	; Mac-driver receive type
MACSRV_SRCROUTE EQU	200h	; Mac-driver supports IBM source routing
MACSRV_RESETDEV EQU	400h	; Mac-driver supports reset-device maccmd
MACSRV_OPENDEV	EQU	800h	; Mac-driver supports open-device and
MACSRV_CLOSEDEV EQU	800h	; (**SAME-BIT**) close-device maccmd
MACSRV_INTR	EQU	1000h	; Mac-driver supports Interrupt command
MACSRV_SRCBRIDGE EQU	 2000h	 ; Mac-driver supports SRC-ROUTE bridge
MACSRV_GDTADDR	EQU	4000h	; GDT Virtual Addresses supported
;------------------------------------------------------------------------------
;		    MAC SERVICE SPECIFIC CHARACTERISTICS
;		       STRUCTURE NAME --- mac_ss_char
;------------------------------------------------------------------------------
mac_ss_char	STRUC
mc_size 	dw	?			; size of mac char
mc_srvtype	db	MAC_SRVTYPE_SZ dup (?)	; mac service type
mc_snpasz	dw	?			; snpa size
mc_permsnpa	db	MAC_SNPA_SZ dup (?)	; permanent SNPA address
mc_cursnpa	db	MAC_SNPA_SZ dup (?)	; current SNPA address
mc_funcaddr	dd	?			; current FUNC address
mc_mcast_pt	dd	?			; multicast table
mc_linkspeed	dd	?			; link speed bits/sec
mc_srvflags	dd	?			; MAC service flags
mc_mxpktsz	dw	?			; max packet size
mc_xmtbufsz	dd	?			; total xmtbuf size
mc_xmtblksz	dw	?			; xmtbuf block size
mc_rcvbufsz	dd	?			; total rcvbuf size
mc_rcvblksz	dw	?			; rcvbuf block size
mc_ieeevendor	db	3 dup (?)		; IEEE vendor code
mc_boardcode	db	?			; Vendor adapter code
mc_serialnum	dd	?			; Vendor serial number
;mc_interrupt	 dw	 ?			   IRQ Level used by adapter
mac_ss_char	ENDS
;------------------------------------------------------------------------------
; MAC STATUS MASKS (ms_status)

MSMSK_OPCODE	EQU	7h				; OPCODE MASK
MSMSK_MACBOUND	EQU	8h				; MAC-BOUND MASK
MSMSK_MACOPEN	EQU	10h				; MAC-OPEN MASK
;------------------------------------------------------------------------------
; VALUES of OPCODE-portion of MAC STATUS WORD (ms_status)
MS_NO_HDW		EQU	0	; No hardware installed
MS_BAD_HDWDIAG		EQU	1	; bad hardware (diag failed)
MS_BAD_HDWCNFG		EQU	2	; bad hardware config
MS_HDW_FAULT		EQU	3	; hardware fault
MS_SFT_FAULT		EQU	4	; software fault
MS_OK			EQU    23	; hardware fully operational
;------------------------------------------------------------------------------
; PACKET FILTER FLAGS (ms_status)
PF_DIR_MCAST		EQU	1h	; directed, mcast/func
PF_BROADCAST		EQU	2h	; broadcast
PF_PROMISCUOUS		EQU	4h	; promiscuous
PF_SRCROUTE		EQU	8h	; src-routing
;------------------------------------------------------------------------------
;			 STRUCTURE NAME --- MStats
;------------------------------------------------------------------------------
;Mac statistics
MStats			STRUC
ms_RxTotPktCT		dd	?	;total frames received
ms_RxCrcPktCT		dd	?	;frames with crc error
ms_RxTotByteCT		dd	?	;total bytes received
ms_RxNobufPktCT 	dd	?	;frames discarded - no buffer space
ms_RxMcastPktCT 	dd	?	;multicast frames received
ms_RxBcastPktCT 	dd	?	;broadcast packets received
ms_RxErrPktCT		dd	?	;frames received with errors
ms_RxBigPktCT		dd	?	;frames exceeding maximum size
ms_RxSmallPktCT 	dd	?	;frames smaller than minimum size
ms_RxMcastByteCT	dd	?	;multicast bytes received
ms_RxBcastByteCT	dd	?	;broadcast bytes received
ms_RxHwerrPktCT 	dd	?	;frames received - hardware error
ms_TxTotPktCT		dd	?	;total frames transmitted
ms_TxTotByteCT		dd	?	;total bytes transmitted
ms_TxMcastPktCT 	dd	?	;multicast packets sent
ms_TxBcastPktCT 	dd	?	;broadcast packets sent
ms_TxBcastByteCT	dd	?	;broadcast bytes transmitted
ms_TxMcastByteCT	dd	?	;multicast bytes sent
ms_TxTimeoutPktCT	dd	?	;frames not transmitted - time out
ms_TxHwerrPktCT 	dd	?	;frames not transmitted - HW err
MStats			ENDS
;------------------------------------------------------------------------------
MAC_STATS_ENTRYCT	EQU	20d
;------------------------------------------------------------------------------
;		      STRUCTURE NAME --- mac_ss_status
;------------------------------------------------------------------------------
; MAC SERVICE SPECIFIC STATUS TABLE
mac_ss_status	STRUC
ms_size 	dw	?		; size of mac status
ms_dt_diag	dd	?		; date-time of last diag
ms_status	dd	?		; MAC status
ms_pktfilter	dw	?		; packet filter
ms_mediastatspt dd	?		; point to media specific stats
ms_dt_clrstats	dd	?		; data/time last clrstats
ms_stats	dd	MAC_STATS_ENTRYCT dup (?)	; statistics array
mac_ss_status	ENDS
;------------------------------------------------------------------------------
;802.3 RECEIVE ERROR FAILURE MASK
RCVCRCERR	EQU	1		;CRC error
RCVFRAMEERR	EQU	2		;Framing Error
RCVTOOLONG	EQU	4		;frame size exceeds maximum

;802.3 TRANSMIT ERROR FAILURE MASK
XMTEXCOLL	EQU	1		;excessive collisions error
XMTCARRIER	EQU	2		;carrier check failed error
XMTSHORTCIRC	EQU	4		;short circuit error
XMTOPENCIRC	EQU	8		;open circuit error
XMTTOOLONG	EQU	10h		;frame too long error
XMTREMFAIL	EQU	20h		;remote failure to defer collision
;------------------------------------------------------------------------------
;		       STRUCTURE NAME --- mac_eth_stats
;------------------------------------------------------------------------------
; 802.3 MEDIA SPECIFIC STATISTICS
mac_eth_stats	STRUC
me_length	dw	?		;length of 802.3 structure
me_vers 	dw	?		;version level
me_alerr	dd	?		;frames with alignment error
me_rcvmsk	dd	?		;receive error failure mask
me_overrun	dd	?		;frames with overrun error
me_1col 	dd	?		;frames xmitted after at least 1 collision
me_def		dd	?		;frames xmitted after deferring
me_16coll	dd	?		;frames not xmitted - 16 collisions
me_totcoll	dd	?		;total collisions during transmission
me_late 	dd	?		;late collisions
me_ex1coll	dd	?		;frames xmitted after exactly 1 collision
me_mulcoll	dd	?		;frames xmitted after multiple collisions
me_heart	dd	?		;frames transmitted, CD heartbeat
me_jabber	dd	?		;jabber errors
me_cslost	dd	?		;carrier sense lost during xmission
me_xmtmask	dd	?		;transmit error failure mask
mac_eth_stats	ENDS
;------------------------------------------------------------------------------
;			STRUCTURE NAME --- mac_udt
;------------------------------------------------------------------------------
; MAC Upper-Dispatch-Table (UDT)
mac_udt 	STRUC
mu_comcharpt	dd	?		; back ptr to com_char
mu_request	dd	?		; MAC Request()
mu_xmtchain	dd	?		; MAC TransmitChain()
mu_xfrdata	dd	?		; MAC TransferData()
mu_rxbuffree	dd	?		; MAC ReceiveRelease()
mu_indon	dd	?		; MAC IndicationOn()
mu_indoff	dd	?		; MAC IndicationOff()
mac_udt 	ENDS
;------------------------------------------------------------------------------
; NETBIOS DRIVER TYPE (nb_drvtype)
NB_NCB_DRVTYPE		EQU	1	; NCB driver type

; NETBIOS Network Status (nb_netstatus)
NB_DRVTYPE_MSK		EQU	2h

; Values of the NB_DRVTYPE_MSK flag
NORMAL_DRIVER		EQU	0000h
LOOPBACK_DRIVER 	EQU	0002h

; PROTO's LDT Interface Flags (pl_intfflags)
NON_LLC_FRAMES		EQU	1h
SPEC_LSAP_FRAMES	EQU	2h
NONSPEC_LSAP_FRAMES	EQU	4h
;------------------------------------------------------------------------------
;		       STRUCTURE NAME --- vecni_ldt
;------------------------------------------------------------------------------
; PROTOCOL Lower-Dispatch-Table (LDT) :
; ===================================
; MAC-drivers use this dispatch table to hand incoming pkts etc.

vecni_ldt	STRUC
pl_comcharpt	dd	?		; back ptr to com_char
pl_intfflags	dd	?		; interface flags
pl_reqconfirm	dd	?		; PROTO RequestConfirm()
pl_xmtconfirm	dd	?		; PROTO TransmitConfirm()
pl_rcvlookahead dd	?		; PROTO ReceiveLookahead()
pl_indcomplete	dd	?		; PROTO IndicationComplete()
pl_rcvchain	dd	?		; PROTO ReceiveChain()
pl_statusind	dd	?		; PROTO StatusIndication()
vecni_ldt	ENDS
;------------------------------------------------------------------------------
; PROTOCOL Upper-Dispatch-Table (UDT) :
; ===================================
; This dispatch table is filled by the NETBIOS driver.
; In the first release of Protocol Manager,
; the binding of the Protocol-Moudle occurs at its lower boundary
; only, and not at the upper boundary.
;
; NOTE :
;	o The "nu_request" field points to NETBIOS's Request handler,
;	  which should adhere to the VECTOR calling conventions!!!
;	o The "nu_ncbhandler" field points to NETBIOS's NCB handler,
;	 which should adhere to the LANMAN calling conventions!!!

;		  ***	STRUCTURE NAME --- nb_udt   ***
nb_udt		STRUC
nu_comcharpt	dd	?		; back ptr to com_char
nu_request	dd	?		; NEBIOS Request()
nu_ncbhandler	dd	?		; NETBIOS NCB Handler
nb_udt		ENDS
;------------------------------------------------------------------------------
;		       STRUCTURE NAME --- com_char
;------------------------------------------------------------------------------
; COMMON CHARACTERISTICS
; ======================
com_char	STRUC
					; OFFSET
cc_size 		dw	?	; 00 : size of CC table
cc_level		dw	?	; 02 : CC table level
cc_ss_level		dw	?	; 04 : SS table level
cc_major_ver		db	?	; 06 : major module version # (BCD)
cc_minor_ver		db	?	; 06 : minor module version # (BCD)
cc_func_flags		dd	?	; 08 : module function flags
cc_module_name		db	VEC_MODNAME_SZ dup (?)
					; 12 : module name (ASCIIZ)
cc_ul_protolevel	db	?	; 28 : proto-level at module's UL
cc_ul_intftype		db	?	; 29 : intf-type at module's UL
cc_ll_protolevel	db	?	; 30 : proto-level at module's LL
cc_ll_intftype		db	?	; 31 : intf-type at module's LL
cc_moduleid		dw	?	; 32 : module ID of this module
cc_dataseg		dw	?	; 34 : module's (RUN-TIME) DS
cc_sysreq		dd	?	; 36 : system req dispatch entrypt
cc_un_char		dd	?	; 40 : srv-specific char
mac_charpt		dd	?	; 44 : srv-specific status
cc_un_udt		dd	?	; 48 : UDT pointer
cc_ldtpt		dd	?	; 52 : VECTOR_NI LDT pointer
cc_reserve1		dd	?	; 56 : Reserved
cc_reserve2		dd	?	; 60 : Reserved
					; TOTAL : 64 bytes
com_char		ENDS
;------------------------------------------------------------------------------
;		       STRUCTURE NAME --- Rdatablk
;------------------------------------------------------------------------------
; VECTOR-style receive BUFFER DESCRIPTORS

; data block
Rdatablk	STRUC
rblk_datalen	dw	?     ; length of data block
rblk_datapt	dd	?     ; far pointer or physical address to data buffer
Rdatablk	ENDS
;------------------------------------------------------------------------------
;		     STRUCTURE NAME --- Tdatablk
;------------------------------------------------------------------------------
; Vector-style transmit/transfer buffer descriptors
Tdatablk	STRUC
tblk_ptrtype	db	?	;type of pointer (0=physical, 2=GDT)
tblk_resbyte	db	?	;reserved byte
tblk_datalen	dw	?	;length of data byte
tblk_datapt	dd	?	;address of data block
Tdatablk	ENDS
;values of the tblk_ptrtype field
PHYSTYPE	equ	0
GDTTYPE 	equ	2
;------------------------------------------------------------------------------
;		       STRUCTURE NAME --- txbufdescr
;------------------------------------------------------------------------------
; Transmit Buffer Descriptor
; ==========================
; NOTE : "tx_immedpt" is a 4-byte Virtual pointer in OS2 or DOS.
;	 "tx_immedlen" could have a maximum value of 64 bytes.
;	  "tx_blkct" could have a maximum value of 8.

txbufdescr	STRUC
tx_immedlen	dw	?		; byte count of immediate data (max 64)
tx_immedpt	dd	?		; far VIRTUAL ptr to immediate data
tx_blkct	dw	?		; count of tx blocks
tx_datablk	db	(Size Tdatablk) dup (?) ; length of first data block
txbufdescr	ENDS

;offsets of other data blocks in TxBufDescr
tx_datablk2	equ	SIZE txbufdescr
tx_datablk3	equ	tx_datablk2 + SIZE Tdatablk
tx_datablk4	equ	tx_datablk3 + SIZE Tdatablk
tx_datablk5	equ	tx_datablk4 + SIZE Tdatablk
tx_datablk6	equ	tx_datablk5 + SIZE Tdatablk
tx_datablk7	equ	tx_datablk6 + SIZE Tdatablk
tx_datablk8	equ	tx_datablk7 + SIZE Tdatablk
;------------------------------------------------------------------------------
;		      STRUCTURE NAME --- tddatadescr
;------------------------------------------------------------------------------
; Transfer Data Buffer Descriptor
; ===============================
; NOTE : "blk_datapt" of td_datablk is a PHYSICAL POINTER in OS2 or DOS.
;	  "td_blkct" could have a maximum value of 8.

tddatadescr	STRUC
td_blkct	dw	?		; count of td blocks
td_datablk	db	(Size Tdatablk) dup(?)	; "td_blkct" number of data blocks
tddatadescr	ENDS
;------------------------------------------------------------------------------
;		     STRUCTURE NAME --- rxbufdescr
;------------------------------------------------------------------------------
; Receive Chain Buffer descriptor
; ===============================
; NOTE : "blk_datapt" of rx_datablk is a VIRTUAL POINTER in OS2 or DOS.
;	  "rx_blkct" could have a maximum value of 8.

rxbufdescr	STRUC
rx_blkct	dw	?		; count of rx datablks
rx_datablk	db	(Size Rdatablk) dup (?) ; "rx_blkct" number of data blocks
rxbufdescr	ENDS
;------------------------------------------------------------------------------
;			STRUCTURE NAME --- Req_Control
;------------------------------------------------------------------------------
Req_Control	 STRUC
Req_Link	dw	?		;link to next Req_Control structure
Req_Free	db	?		;buffer free flag
					;1:free to be filled, 0:available
Req_Handle	dw	?		;handle of tranmit request
Req_ProtID	dw	?		;protocol ID of transmitter
Req_Control	 ENDS

Req_Avail	EQU	1		;Buffer is available
Req_Used	EQU	0		;buffer is not available
ReqBufferNo	EQU	40

;------------------------------------------------------------------------------
;			STRUCTURE NAME --- Tx_Control
;------------------------------------------------------------------------------

Tx_Control	STRUC
TCS_Link	dw	?		;link to next Tx_Control structure
TCS_Free	db	?		;buffer free flag
					;1:free to be filled, 0:available
TCS_Page	db	?		;TxMit page (0 - 3)
TCS_PAddr	dd	?

TCS_Size	dw	?		;size of packet to be transmitted
TCS_Handle	dw	?		;handle of tranmit request
TCS_ProtID	dw	?		;protocol ID of transmitter

TCS_PStart	dw	?		;page number of TX buffer on board
Tx_Control	ENDS

TCS_Avail	EQU	1		;Buffer is available
TCS_Used	EQU	0		;buffer is not available
