COMMENT  ^

This module deals with interrupts for the driver. It contains the
interrupt handler, as well as routines which get the interrupt
vector and test the interrupt.

^
;------------------------------------------------------------------------------
;                              INCLUDE FILES
;------------------------------------------------------------------------------
INCLUDE macros.inc
INCLUDE vector.inc
INCLUDE os2defs.inc
INCLUDE dp8139.inc
INCLUDE RTL8139.inc
include bit.inc
IFNDEF  DOS                             ;286 protected mode needed to
.286P                                   ;examine machine status word
ENDIF                                   ;under OS/2

;===============================================================================
DATASEG_BEG

EXTRN   DriverHead:WORD
EXTRN   CommChar:WORD
EXTRN   indicate:WORD, ProtDS:WORD
EXTRN   Indoffs:BYTE, IndicationComplete:DWORD
EXTRN   RequestConfirm:DWORD
EXTRN   TCSNextToXmit :WORD,NICCMD:WORD,NICISR:word,NICIMR:word, NICIDR0:word,NICTCR:word
EXTRN   wait_pci:NEAR, set_param:NEAR

EXTRN   IRQ:WORD,force_flag:byte,Medium:byte
PUBLIC  IntSem
PUBLIC FOR_RET_FLAG
FOR_RET_FLAG    DW      0

public  Status
Status          dd      ?
IrqNum          dw      ?               ;interrupt level of board
IntSem          dw      0               ;semaphore for interrupt routine
IntFlag         dw      0               ;flag for testing interrupt level

IntRoutine      dw      ?               ;vector to interrupt routine used

IFDEF   DOS                             ;DOS
DosStack        dw    256 dup (?)     ;stack for DOS interrupts
EndStack        dw      ?               ;end of stack
ENDIF

;---------------------------------------
Position        dw      0
Attribute       db      0ch,00eh
EXTRN   Indoffs:BYTE
EXTRN   IndReqF :byte
EXTRN   Device_Help:DWORD
;---------------------------------------
DATASEG_END
;===============================================================================
CODESEG_BEG

EXTRN   Give_Packets:near , Xmit_Done:near  , Overflow_Process:near
EXTRN   ReadCounters:NEAR , IOBase:WORD

fptr            dw  2 dup (?)
fptrlocation    dw  ?




COMMENT ^

----------------------------------------------------------------------------
                    LOCAL PROCEDURE NAME --- Dismiss
$Name: Dismiss - dismiss the interrupt and disable further interrupts$
----------------------------------------------------------------------------
Registers used:
        AX and DX

$Description:
This calls EOI to dismiss the interrupt at the 8259.
$

$Method:
$
^
L_PROC          Dismiss
IFDEF           DOS
                mov     al,EOI          ;give EOI to second PIC if necessary
                cmp     cs:irqnum,7
                jle     @F
                mov     dx,PIC2COM
                out     dx,al
@@:             mov     dx,PIC1COM      ;give EOI to first PIC
                out     dx,al
ELSE
                mov     ax,irqnum       ;dismiss the interrupt at the PIC
                mov     dl,DEVHLP_EOI
                call    [Device_Help]
ENDIF
                ret
L_ENDP          Dismiss
COMMENT ^

----------------------------------------------------------------------------
                    PUBLIC PROCEDURE NAME --- IntCheck
$Name: IntCheck - Check that interrupt works$
----------------------------------------------------------------------------
$Description:
This routine is used to make sure that our interrupt works. When enterred,
some bit must be set in the ISR. This is usually done by the loopback test.
This routine enables the board interrupts. This should cause the test
interrupt routine to be enterred and to set the intflag. This routine
polls that flag. If no interrupt is received, it times out. It returns 0
in ax if an interrupt is received, 1 if it is not.

The interrupt routine will disable board interrupts again.
$

$Method:
$
^
A_PROC          IntCheck

SawInt:         mov     ax, SUCCESS   ;success
                ret
A_ENDP          IntCheck


;----------------------------------------------------------------------------
;the following macros are needed to make the interrupt routine compatible
;with DOS
;----------------------------------------------------------------------------
;                        MACRO NAME --- PUSHALL
;----------------------------------------------------------------------------
PUSHALL         MACRO
IFDEF           DOS                             ;DOS
                mov     CS:[EndStack],ss        ;switch stacks
                mov     CS:[EndStack-2],sp      ;save original stack parameters
                lea     ax,[EndStack-4]
                mov     sp,ax                   ;set new stack (sp)
                mov     ax,cs
                mov     ss,ax                   ;set new stack segment (ss)

                ;flags, ax, dx saved earlier
                push    bx
                push    cx
                push    si
                push    di
                push    ds
                push    es
ENDIF
                ENDM
;----------------------------------------------------------------------------
;                        MACRO NAME --- POPALL
;----------------------------------------------------------------------------
POPALL          MACRO
IFDEF           DOS                             ;DOS
                pop     es
                pop     ds
                pop     di
                pop     si
                pop     cx
                pop     bx

                mov     sp,CS:[EndStack-2]      ;switch back stack
                mov     ss,cs:[EndStack]        ;restore ss,sp
ENDIF
                ENDM
;----------------------------------------------------------------------------
;                        MACRO NAME --- INTENTER
;----------------------------------------------------------------------------
INTENTER        MACRO
IFDEF           DOS                             ;DOS
                pushf
                push    eax
                push    ebx
                push    ecx
                push    edx
ENDIF
                ENDM
;----------------------------------------------------------------------------
;                        MACRO NAME --- INTRET
;----------------------------------------------------------------------------
INTRET          MACRO
IFDEF           DOS                             ;DOS
                pop     edx
                pop     ecx
                pop     ebx
                pop     eax
                popf
                iret
ELSE
                retf
ENDIF
                ENDM
;----------------------------------------------------------------------------
COMMENT ^

$Name: Interrupt - main interrupt handling routine$

$Description:
Board interrupts are disabled while this routine runs. In addition, the
routine is semaphore protected to prevent reentry. This routine does the
following:

        1. Processes any received packets.
        2. Processes an overflows.
        3. Processes any completed transmissions.
        4. Repeat 1-3 until the ISR has been cleared.

$

$Method:
In DOS, ax, dx, and the flags are pushed on the calling stack. Only these
may be used until stacks are switched.
$
^
;              ***  PROCEDURE NAME --- Interrupt  ***
;FLOWCHART:
;                       Interrupt enter
;                              |
;                             \|/
;                       Confirm reenter ? (check semaphore, 0 - not
;                              |                            1 - yes)
;                             \|/
;                       Push all flags
;                              |
;                             \|/
;                       Handle interrupt
;                   ___________|
;                  |          \|/
;                  |    Give_Packets      <- handle receive
;                  |           |
;                  |          \|/
;                  |    Overflow_Process  <- Handle overflow
;                  |           |
;                  |          \|/
;                  |    Transmit done or  <- Handle transmit
;                  |         error
;                  |           |
;                  |          \|/
;                  |    ReadCounter       <- Handle read counter
;                  |           |
;                  |__________\|/
;               some interrupt |
;                    left      |
;                             \|/
;                       Release semaphore
;                              |
;                             \|/
;                       Pop all flags
;                              |
;                             \|/
;                       Return from interrupt
;

L_PROC          Interrupt, FAR
IFDEF           DOS                       ;DOS
ASSUME  DS:NOTHING
ENDIF
                cli
                INTENTER                  ;save ax,dx,flags in DOS

;dismiss at the PIC
;ensure that this code is not reentered
SemSet:         mov     al,1              ;set AL=1
                XOR     AH, AH            ;clear AH = 0
                xchg    ax,IntSem         ;exchange ax; Default=0
                cmp     ax,0              ;check semaphore = 0 ?
                je      @F                ;could do the job of interrupt

Next_Interrupt:


IFDEF           DOS                       ;DOS reentry

                pop     edx
                pop     ecx
                pop     ebx
                pop     eax
                popf

                jmp   DWORD PTR CS:fptr   ;jmp old int vector services
ELSE
                stc                       ;OS/2 set carry flag = 1
                                          ;service next victor for OS/2
                jmp     Debug_ISR_Exit
ENDIF

;Process the interrupts
                                          ;bx,cx,si,di,ds,es

@@:
OurInterrupt:
                PUSHALL                   ;switch to a new stack and reserve
                                          ;bx,cx,si,di,ds,es
                WR_NIC  IMR,0             ;disable all board interrupts
                RD_NIC  BX,ISR            ;get interrupt status reg
                test    BX, 3FH           ;any interrupts left?
                jnz     @F                ;ack 8259 a interrupt
                mov     ax, 0
                xchg    ax, IntSem


                POPALL
                WR_NIC  IMR,3FH                 ;enable interrupts
                jmp     Next_Interrupt
@@:
                call    Dismiss


IFDEF   DOS                               ;DOS
                push    cs                ;DOS version runs in 1 segment
                pop     ds                ;set ds is equal to cs
                ASSUME  DS:DGROUP         ;assume DS:DGROUP
ENDIF

IFNDEF          DOS                       ;OS/2

;****************************** <OS/2> ******************************
;switch to protected mode if necessary
;push 0 on the stack if we enterred the interrupt in real mode, 1 if we
;enterred in protected mode. We use this to see if we should switch back
;to real mode when we exit.

                smsw    ax                      ;see if we're in prot mode
                test    ax,1                    ;test PE bit
                jz      @F
                pushc   1                       ;we were in protected mode
                jmp     intstart
@@:             push    si
                mov     si, OFFSET DriverHead
                mov     dl,DEVHLP_REALTOPROT    ;switch to protected mode
                call    ds:[Device_Help]
                pop     si
                pushc   0
;****************************** <OS/2> ******************************
ENDIF

intstart:
                cld                             ;copies go up DF=0
                mov     IntFlag,1               ;note routine enterred

;main loop of interrupt routine
intloop:                                        ;enable interrupts again
                STI
                mov     DX,DS:NICISR
                in      ax,DX
;;;;;;;;;;;;;;;8139 v200;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
intloop1:
                STI                     ;SID
                push ax

;;;{{{{;;;;;8139 v200;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
isr_test_link_change:
       ;int 3
        test    ax, ISR_PUN
        jz      isr_no_link_change

;       loadport
;       setport RTL_74

       ;int 3
        mov     dx, (DS:NICIDR0)
        add     dx, 074h

        in      ax,dx

       ;int 3

        test    ax,SIZE16_BIT11                 ;link change bit ?
        jz      isr_no_link_change_inisr

;*******************************************
;******* 04/10/1998 add N Way force ********
;*******************************************

        cmp     force_flag,0
        jne     @F
        jmp     N_Way_flag_false

@@:
        test    ax,SIZE16_BIT10
        jz      isr_8139_set_default            ;link down set parameter

;****** Link OK ***************

        mov     dx, (DS:NICIDR0)
        add     dx, 062h
        in      ax,dx
        test    ax,SIZE16_BIT12
        jnz     Dump_Hub
        mov     ax,1200h
        out     dx,ax
        push    eax
        mov     eax,(033333333d)*2      ;2.0 sec  delay
        call    wait_pci
        pop     eax

        mov     dx,(DS:NICISR)
        mov     ax,20h                 ;clear link change bit
        out     dx,ax


Dump_Hub:
        mov     dx, (DS:NICIDR0)
        add     dx, 06Ah
        in      ax,dx
        test    ax,SIZE16_BIT0
        jnz     No_Dump_Hub

        cmp     Medium,1
        jne     @F
        mov     ax,0000h                ;10half
        jmp     test_link_ok_2
@@:
        cmp     Medium,2
        jne     @F
        mov     ax,0100h                ;10full
        jmp     test_link_ok_2

@@:
        cmp     Medium,3
        jne     @F
        mov     ax,2000h                ;100half
        jmp     test_link_ok_2
@@:
        cmp     Medium,4
        jne     @F
        mov     ax,2100h                ;100full

test_link_ok_2:

        mov     dx,ds:nicIDR0
        add     dx,62h
        out     dx,ax

@@:

No_Dump_Hub:

N_Way_flag_false:
;*******************************************
;******* 04/10/1998 add N Way force ********
;*******************************************
        mov     dx, (DS:NICIDR0)
        add     dx, 074h

        in      ax,dx
        test    ax,SIZE16_BIT10
        jz      isr_8139_set_default

        ;;;; link on;;;;;;;;;;;
 ;      SHOW_EVENT      'o'
 ;      SHOW_EVENT      'n'
 ifdef debug5
        push    ax
        mov     al, 'O'
;       call    show_char
        mov     al, 'N'
;       call    show_char
        pop     ax
 endif
        ;;;; link on;;;;;;;;;;;



        mov     ax, 003c0h              ;enable Turbo link
        mov     dx, (DS:NICIDR0)
        add     dx, 074h
        out     dx, ax

       ;int 3
        ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
        ;;;;;;delay 100MS;;;;;;;;;;;;;;;;;;
        ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
        push    eax
        mov     eax,033333333d/10d   ;1/10 sec  delay
        call wait_pci
        pop     eax

 ;      loadport
 ;      setport RTL_74

        mov     dx, (DS:NICIDR0)
        add     dx, 074h

        in      ax,dx
        and     ax, (SIZE16_BIT12+SIZE16_BIT13+SIZE16_BIT14)
        cmp     ax, 0
        jnz     isr_next_v200_1
 ;      SHOW_EVENT      'a'
 ifdef debug5
        push    ax
        mov     al, '0'
;       call    show_char
        pop     ax
 endif

        mov     bx,0
        call    set_param

        jmp     isr_con_step4_v200
isr_next_v200_1:

        cmp     ax, SIZE16_BIT12
        jnz     isr_next_v200_3
 ;      SHOW_EVENT      'b'
 ifdef debug5
        push    ax
        mov     al, '1'
;       call    show_char
        pop     ax
 endif
        mov     bx,1
        call    set_param

        jmp     isr_con_step4_v200
isr_next_v200_3:

        cmp     ax,(SIZE16_BIT12+SIZE16_BIT13)
        jnz     isr_next_v200_7
  ;     SHOW_EVENT      'c'
 ifdef debug5
        push    ax
        mov     al, '3'
;       call    show_char
        pop     ax
 endif
        mov     bx,3
        call    set_param

        jmp     isr_con_step4_v200
isr_next_v200_7:

        cmp     ax,(SIZE16_BIT12+SIZE16_BIT13+SIZE16_BIT14)
        jnz     isr_next_v200_8
  ;     SHOW_EVENT      'd'
 ifdef debug5
        push    ax
        mov     al, '7'
;       call    show_char
        pop     ax
 endif
        mov     bx,7
        call    set_param

        jmp     isr_con_step4_v200
isr_next_v200_8:
  ;     SHOW_EVENT      'e'
  ;     SHOW_EVENT      'r'
  ;     SHOW_EVENT      'r'
 ifdef debug5
        push    ax
        mov     al, 'E'
;       call    show_char
        mov     al, 'R'
;       call    show_char
        mov     al, 'R'
;       call    show_char
        pop     ax
 endif
        mov     bx,7
        call    set_param
        jmp     isr_con_step4_v200

isr_8139_set_default:
        ;;;; link down;;;;;;;;;;;
   ;    SHOW_EVENT      'd'
   ;    SHOW_EVENT      'n'
 ifdef debug5
        push    ax
        mov     al, 'D'
;       call    show_char
        mov     al, 'N'
;       call    show_char
        pop     ax
 endif
        ;;;; link down;;;;;;;;;;;

   ;    loadport
   ;    setport RTL_70

        mov     dx, (DS:NICIDR0)
        add     dx, 070h

        push    eax
        mov     eax, PARAM70
        out     dx,eax


   ;    loadport
   ;    setport RTL_78

        mov     dx, (DS:NICIDR0)
        add     dx, 078h

        mov     eax,PARAM78
        out     dx,eax

   ;    loadport
   ;    setport RTL_7C

        mov     dx, (DS:NICIDR0)
        add     dx, 07ch

        mov     eax, PARAM7C
        out     dx,eax

        pop     eax

        jmp     isr_test_link_change_exit
isr_no_link_change_inisr:
    ;   SHOW_EVENT      'q'
 ifdef debug5
        push    ax
        mov     al, 'Q'
;       call    show_char
        pop     ax
 endif
isr_test_link_change_exit:


       ;int 3
isr_con_step4_v200:
;;;}}}}};;;;8139 v200;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
                mov     ax,ISR_PUN
                mov     DX,DS:NICISR
                out     dx,ax
isr_no_link_change:
                pop  ax
;;;;;;;;;;;;;;;8139 v200;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@:
        mov     DX,DS:NICISR
        in      ax,DX
        test    al,ISR_ROK+ISR_RER+ISR_RXOVW ;Receive ok,err,ovw
        jz      chkter                      ;no anyone set
        call    Give_Packets            ;process receives
        mov     DX,DS:NICISR
        in      ax,DX

chkter:
        test  al,ISR_TER             ;transmit done or error?
        jz    @F

        ;  push    ax
        ;  mov     al,'A'
        ;  call    show_char1
        ;  pop     ax

        push     ax
        mov   dx,DS:NICISR
        mov   al,ISR_TER
        out   dx,al

        mov   dx,DS:NICTCR
        mov   al,01h
        out   dx,al                  ;Clear Abort

        pop     ax

@@:     test    al,ISR_TOK              ;transmit done ?
        jz      @F                      ;no
        mov   dx,DS:NICISR              ;;SID
        mov   al,ISR_TOK                ;;SID
        out   dx,al                     ;;SID
        call    Xmit_Done               ;yes. process it.

@@:
        CLI                             ;turn off interrupts. Otherwise, they might nest

        mov     DX,DS:NICISR
        in      ax,DX
        test    ax,3fh                 ;any interrupts left?
                                       ;ROK,RER,TOK,TER,RXOVW,PUN
        jz      intdone                 ;yes. process them.
        jmp     intloop1


intdone:

IFNDEF    DOS                             ;OS/2, DOS not bimodal
          call    RequestResponse
ENDIF
          WR_NIC  IMR,03fh                ;enable interrupts

testexit: mov     IntSem,0                ;release the semaphore

;****************************** <OS/2> ******************************
IFNDEF          DOS                             ;OS/2, DOS not bimodal
                pop     ax                      ;get back processor mode
                cmp     ax,0                    ;were we in real mode when we entered?
                jne     @F                      ;no

;switch back to real mode
                mov     si,OFFSET DriverHead
                mov     dl,DEVHLP_PROTTOREAL  ;switch to real mode
                call    ds:[Device_Help]
;****************************** <OS/2> ******************************
ENDIF

@@:             clc                           ;indicate that interrupt was ours

                POPALL
Debug_ISR_Exit:
                INTRET
L_ENDP          Interrupt

;----------------------------------------------------------------
;Input  DS == DGROUP
;Output
A_PROC          RequestResponse
        ;----- Give status to protocol stack
        cmp     IndReqF,0
        jne     RequestResponse_1
        jmp     RequestResponse_ret

RequestResponse_1:
        mov     dx,4                    ;Set Status code
        call    ReturnStatusIndication
        mov     IndReqF,0               ;Reset flag

        cmp     IndOffs,0
        je      RequestResponse_3
        jmp     RequestResponse_ret

RequestResponse_3:
        push    CommChar.cc_moduleid    ;Args for IndComplete
        push    ProtDS
        sti
        call    [IndicationComplete]
        cli
RequestResponse_ret:
        ret
A_ENDP          RequestResponse
;----------------------------------------------------------------
;Input  DX == Indication Code
;Output
;
A_PROC          ReturnStatusIndication
        pushf
        push    es

        mov     Indicate,0FFh           ;Protocol will reset this to keep inds off
        inc     IndOffs                 ;Turn off indications before

        push    CommChar.cc_moduleid    ;Args for IndComplete
        xor     ax,ax
        push    ax
        push    ds                      ;Address if indicate byte
        pushc   <OFFSET Indicate>
        push    dx                      ;Pass Operation Code
        push    ProtDS                  ;Pass protocol DS
        sti
        call    [Status]
        cmp     Indicate, 0             ;Turn on indications
        je      @F                      ;No
        dec     IndOffs                 ;Yes
@@:
        pop     es
        popf
        ret
A_ENDP          ReturnStatusIndication
COMMENT ^

----------------------------------------------------------------------------
$Name: SetInterrupt - Process Interrupts$
----------------------------------------------------------------------------
$Description:
This routine switches the IntRoutine variable so the whole interrupt
routine is executed rather than just the test code. This is called at
bind time.
$

$Method:
$
^
;           ***  PROCEDURE NAME --- SetInterrupt  ***

PUBLIC          SetInterrupt
L_PROC          SetInterrupt
IFDEF DOS                               ;DOS
                mov     IntRoutine, Offset DGROUP:intloop
ELSE                                    ;OS/2
;****************************** <OS/2> ******************************
                mov     IntRoutine, Offset CGROUP:intloop
;****************************** <OS/2> ******************************
ENDIF

                ret
L_ENDP          SetInterrupt

;------------------------------------------------------------------------------


;------------------------------------------------------------------------------

COMMENT ^

----------------------------------------------------------------------------
$Name: GetInt - set up the interrupt vector$
----------------------------------------------------------------------------

$Description:
This routine sets up our interrupt vector. Under OS/2, this is just a
DEVHLP call. Under DOS, we must do it by hand. If this is a microchannel card,
special code is needed to support interrupt sharing. Under OS/2, this
is just a different call. Under DOS, we must add it to the chain of interrupts.
See IBM PS/2 BIOS Tech Ref section 4 for info on the interrupt chain.
The microchannel code is left in here for future uses. It is not used
for the board.
$

$Method:
$
^
;                  ***  PROCEDUR NAME --- GetInt  ***

L_PROC          GetInt
IntVal          EQU     <WORD PTR [bp-2]>
;               Stack status :
;
;                  |   es   |
;            sp ___|   ds   |
;                  |        |___ bp
;                  |   bp   |
;                  +--------+
;
                push    bp          ;reserve bp
                mov     bp,sp       ;set bp is equal to sp
                sub     sp,2        ;decrease stack pointer

IFDEF           DOS                 ;DOS
                ;get vector
                push    ds          ;reserve ds
                push    es          ;reserve es
                cli                 ;clear interrupt
                mov     ax, irqnum
                cmp     al,7        ;convert hw int to sw int
                jg      @F          ;the conversion depends on which PIC
                add     al,8        ;the interrupt is on.
                jmp     intset
@@:             add     al,68h
intset:         mov     IntVal,ax   ;store software int level

setvctr:
                push    es
                push    ds
                push    si

                mov     ax,IntVal               ;get interrupt level
                mov     ah,35h                  ;get interrupt vector
                int     21h                     ;call DOS
                mov     si, offset DGROUP:fptr  ;forward pointer

                mov     [si],bx                 ;store old interrupt
                mov     [si+2],es               ;vector
                pop     si
                pop     ds
                pop     es

                mov     dx,OFFSET DGROUP:Interrupt ;interrupt routine
                mov     ax,IntVal               ;get interrupt level
                mov     ah,25h                  ;set interrupt vector
                push    cs                      ;make ds is equal to
                pop     ds                      ;cs
                int     21h                     ;call DOS
                pop     es                      ;restore es, ds
                pop     ds

;unmask the interrupt at the PIC
                mov     cx,irqnum               ;interrupt level
                mov     dx,PIC1MASK
                cmp     cl,7                    ;is it on the fisrt PIC?
                jle     @F                      ;yes
                sub     cl,8                    ;get level on PIC 2
                mov     dx,PIC2MASK
@@:             in      al,dx                   ;get current mask
                jmp     @F                      ;delay
@@:             xor     ch,ch                   ;get a mask with all 1s
                                                ;except for this interrupt
                mov     bl,1
                shl     bl,cl
                not     bl
                and     al,bl
                out     dx,al           ;unmask the interrupt
                sti
ELSE

;OS/2 routines

;interrupt level 2 comes in on interrupt level 9 at the PIC due to the
;PIC cascading. DOS takes care of this for you, but OS/2 does not. Hence,
;we alias level 2 to level 9.
                cmp     irqnum,2
                jne     @F
                mov     irqnum,9

@@:
;set up IRQ vectors. If this is a PS/2, try first to get the interrupt shared.
                mov     ax,OFFSET Interrupt     ;set up our interrupt vector
                mov     bx,irqnum               ;interrupt level
                mov     dh,1                    ;have share interrupt
                mov     dl,DEVHLP_SETIRQ
                call    ds:[Device_Help]

                jnc     GotInt                  ;is there an error?

                mov     ax,OFFSET Interrupt     ;set up our interrupt vector
                mov     bx,irqnum               ;interrupt level
                mov     dh,0                    ;don't share interrupt
                mov     dl,DEVHLP_SETIRQ
                call    ds:[Device_Help]

                add     sp,2
                pop     bp
                mov     ax,SUCCESS
                ret
ENDIF

GotInt:         add     sp,2
                pop     bp
                mov     ax,SUCCESS
                ret



L_ENDP          GetInt

COMMENT ^

----------------------------------------------------------------------------
$Name: Init_Int - Initialize the interrupt module
----------------------------------------------------------------------------
$Description:
This routine initializes the interrupt routine. It gets the vector
and sets up the routine in test mode. This mode is used by IntCheck to make
sure we can get the interrupt.
$

$Method:
$
^
;                 ***  PROCEDURE NAME --- Init_Int  ***

PUBLIC          Init_Int
L_PROC          Init_Int

IFDEF DOS                             ;DOS
;This IntRoutine is used for IntCheck
                mov     IntRoutine, Offset DGROUP:testexit
ELSE                                  ;OS/2
                mov     IntRoutine, Offset CGROUP:testexit
ENDIF
                mov     ax,IRQ        ;set interrupt level
                mov     IrqNum,ax
                call    GetInt        ;setup up the interrupt vector
                ret
L_ENDP          Init_Int
CODESEG_END
END
