COMMENT  ^

This file holds the routines for dealing with general requests from the
protocol.  All requests enter at the Request procedure and are vectored
from there depending on their Opcode.

The request handler routines
may modify whatever registers they wish, as the Request procedure restores
everything.  The Request procedure also sets up the stack so the handlers
can access the parameters if they wish.  The handlers return their results
in ax.

^
INCLUDE macros.inc
INCLUDE vector.inc
INCLUDE os2defs.inc
INCLUDE dp8139.inc
INCLUDE RTL8139.inc

IFNDEF	DOS				;286 protected mode needed to
.286P					;examine machine status word
ENDIF					;under OS/2


;==============================================================================
;==============================================================================
DATASEG_BEG

EXTRN	ServChar:WORD, CommChar:WORD, LookAheadValue:WORD
EXTRN	ServStat:WORD, MacStat:WORD, MedStat:WORD
EXTRN	DriverHead:WORD, ProtDS:WORD
EXTRN	indicate:WORD
EXTRN	TCSNextToFill:WORD
EXTRN	TCSNextToXmit :WORD


EXTRN	IntSem :word
PUBLIC	IndReqF
IndReqF 	db	0



FirstTime	dw	1		;flag for first SetPacketFilter
PUBLIC RequestConfirm
RequestConfirm	dd	?		;Vector to protocol's RequestConfirm

PUBLIC	IRQ, RemoteBuffer, BrdNum,NetAddr,NetAddress,NetFlag
IRQ		dw	3		;IRQ for adapter
BrdNum		db	1		;0 represent only one board
RemoteBuffer	db	50 dup (?)	;Remote buffer
NetAddr 	db	30 dup (?)
NetAddress	db	30 dup (?)
NetFlag 	db	0

;------------------------------------------------------------------------------
;			  PUBLIC MULTICAST TABLE
;------------------------------------------------------------------------------
;Multicast table
PUBLIC	MultiAddr
MultiAddr	mcast_tbl	<MCASTSZ,0,>
		db	(MCASTSZ-1) * MAC_SNPA_SZ dup (?)
;------------------------------------------------------------------------------
SysInfoSeg	dw	?		;system information segment
;------------------------------------------------------------------------------
;Request dispatch table

IFDEF DOS						;DOS
dispatch	dw	DGROUP:NotSup			;diagnostics
		dw	DGROUP:NotSup			;read error log
		dw	DGROUP:SetStationAddress
		dw	DGROUP:OpenAdapter		;OpenAdaptor
		dw	DGROUP:NotSup			;CloseAdaptor
		dw	DGROUP:NotSup			;ResetMAC
		dw	DGROUP:SetPacketFilter
		dw	DGROUP:AddMulticastAddress
		dw	DGROUP:DeleteMulticastAddress
		dw	DGROUP:UpdateStatistics
		dw	DGROUP:ClearStatistics
		dw	DGROUP:InterruptRequest 	;Interrupt
		dw	DGROUP:NotSup			;Set Functional Address
		dw	DGROUP:SetLookAhead
ELSE							;OS/2
		;************************ <OS/2> **************************
dispatch	dw	CGROUP:NotSup			;diagnostics
		dw	CGROUP:NotSup			;read error log
		dw	CGROUP:SetStationAddress
		dw	CGROUP:OpenAdapter		;OpenAdaptor
		dw	CGROUP:NotSup			;CloseAdaptor
		dw	CGROUP:NotSup			;ResetMAC
		dw	CGROUP:SetPacketFilter
		dw	CGROUP:AddMulticastAddress
		dw	CGROUP:DeleteMulticastAddress
		dw	CGROUP:UpdateStatistics
		dw	CGROUP:ClearStatistics
		dw	CGROUP:InterruptRequest 	;Interrupt
		dw	CGROUP:NotSup			;Set Functional Address
		dw	CGROUP:SetLookAhead
		;************************ <OS/2> **************************
ENDIF
;------------------------------------------------------------------------------
EXTRN	Device_Help:DWORD
DATASEG_END
;==============================================================================

CODESEG_BEG
EXTRN	 IndicationOn:NEAR
;EXTRN	 Device_Help:DWORD
EXTRN	Start_Xmit_1	:near

PUBLIC	IOBase

IOBase		dw     ?	     ;NIC register

;common arguments (to Request)
ProtID		EQU	<WORD PTR [bp+18d]>
ReqHandle	EQU	<WORD PTR [bp+16d]>
Opcode		EQU	<WORD PTR [bp+8d]>
MacDS		EQU	<WORD PTR [bp+6]>
ModDS		EQU	<WORD PTR [bp-2]>
;
;Request stack status :
;		    ___ bp-2
; |	 ds	   |___ bp
; |	 bp	   |	      ProtID	:Module ID of Protocol or 0
; |  Call Request  |___ bp+6  ReqHandle :Unique handle for this request or 0
; |   MACDS	2  |___ bp+8  Param1	:Request dependent word parameter or 0
; |   Opcode	2  |___ bp+10 Param2	:Request dependent word parameter or 1
; |   Param2	4  |	      Opcode	:Opcode of request
; |   Param1	2  |___ bp+16 MACDS	:DS of called MAC module
; |  ReqHandle	2  |___ bp+18
; |   ProtID	2  |
; +----------------+
;
L_PROC OpenAdapter
		mov	ax, SUCCESS
		ret
L_ENDP OpenAdapter
;----------------------------------------------------------------

L_PROC		InterruptRequest
	pushf
	cli
	mov	IndReqF,1		;Request an interrupt "indication".
InterruptRequest_1:
	cmp	IntSem,0		;
	je	@F
	jmp	InterruptRequest_end	;Jump if Interrupt service routine
@@:
;------ Transmit one packet to get transmitcompleted interrupt
	mov	ax,TCSNextToFill	;Get next Tx_Control to fill
	cmp	ax,TCSNextToXmit
	je	@F
	;--- Any TX packet pending on TCS
	jmp	InterruptRequest_end
@@:
      ; call	TransmitDummy

InterruptRequest_end:
	popf
	mov	ax,REQUEST_QUEUED
	ret
L_ENDP		InterruptRequest
;----------------------------------------------------------------
;Input	TCSNextToFill == TCSNextToXmit
;	DS == CGROUP
;Ouput
A_PROC		TransmitDummy
	push	di

	mov	di,TCSNextToFill	;Get next Tx_Control to fill
	mov	al,TCS_USED		;Mark as used
	xchg	[di].TCS_Free,al

@@:	mov	ax,[di].TCS_Link	;Update pointer to next Tx_control
	mov	TCSNextToFill,ax	;Refresh next TCS to fill
	xor	ax,ax
	mov	[di].TCS_Handle,ax
	mov	ax,ProtId		;Store protocol ID
	mov	[di].TCS_ProtId,ax
	xor	ax,ax			;Set byte count == 0
	mov	[di].TCS_Size,ax
	mov	bx,TCSNextToXmit	;Next TCS to send
	call	Start_Xmit_1

	pop	di
	ret
L_ENDP		TransmitDummy
;----------------------------------------------------------------
L_PROC SetFunctionalAddress
       mov ax,SUCCESS
       ret
L_ENDP SetFunctionalAddress

COMMENT ^
-------------------------------------------------------------------------
$Name: NotSup - deal with requests which are not supported$
-------------------------------------------------------------------------
$Description:
This procedure returns with NOT_SUPPORTED in AX.
$

$Method:
$
^
;		    ***  PROCEDURE NAME --- NotSup  ***

L_PROC		NotSup
		mov	ax,NOT_SUPPORTED
		ret
L_ENDP		NotSup

COMMENT ^
-------------------------------------------------------------------------
$Name: SetStationAddress - Set the network address of the station$
-------------------------------------------------------------------------
$Description:
This sets the address in the PAR registers on board and then sets in the
characteristics table.
$

$Method:
$
^
;	    ***  PROCEDUR ENAME --- SetStationAddress  ***
;95.12.13
L_PROC		SetStationAddress

AdaptAddr	EQU	<DWORD PTR [bp+10d]>
;Param1 : Pad parameter - must be zero
;Param2 : Buffer containing the adapter address
;set up the address
		pushf			   ;reserve flags
		cli			   ;clear interrupt
		push	ds
		lds	si,AdaptAddr	   ;get pointer to address
		mov	ax,[si] 	   ;set addr. on NIC
		WR_NIC	IDR0,ax
		mov	ax,[si]+2
		WR_NIC	IDR2,ax
		mov	ax,[si]+4
		WR_NIC	IDR4,ax

		pop	es		   ;get our DS into ES

;Here ! Rapus modified it
		lds	si,Adaptaddr
		mov	di,OFFSET ES:ServChar.mc_cursnpa
					   ;offset of table
		mov	cx,3		   ;address is 3 words
		rep	movsw		   ;copy address
		popf

		mov	ax, SUCCESS
		ret

L_ENDP		SetStationAddress

COMMENT ^
-------------------------------------------------------------------------
$Name: SetPacketFilter - set the packet filter$
-------------------------------------------------------------------------
$Description:
This procedure sets the packet filter and sets the appropriate bits on the
board for this filter.
$

$Method:
$
^
;		***  PROCEDURE NAME --- SetPacketFilter  ***

L_PROC		SetPacketFilter
;Param1 : FilterMask - Bit mask for packet filter
;Param2 : Pad parameter - must be 0

FilterMask	EQU		<WORD PTR [bp+14d]>

		pushf				;reserve flags
		cli				;clear interrupt
		mov	ax,FilterMask		;get the new filter
		test	ax, 0FFFFh - 7
		;do we support it (only first three bits can be 1)
		jz	@F			;yes, match our function
		mov	ax,INVALID_PARAMETER	;no. return error
		popf				;restore flags
		ret
@@:		cmp	FirstTime,1		;is this the first time called
		jne	@F			;no
		mov	FirstTime,0		;mark as has been called
		push	ax			;save ax
		push	ds			;set up for FAR call
		push	cs
		call	IndicationOn		;call indication
		pop	ax			;restore FilterMask
@@:		mov	bl,0
		test	ax, PF_DIR_MCAST	;accept multicast?
		jz	@F
		or	bl,RCR_AM		;yes. set bit.
@@:		test	ax,PF_BROADCAST 	;accept broadcast?
		jz	@F			;no
		or	bl,RCR_AB		;yes. set bit
@@:		test	ax,PF_PROMISCUOUS	;promisuous receive mode?
		jz	@F			;no
		xor	bh,bh
		or	bl,RCR_AAP+RCR_AB+RCR_AM  ;yes. set bits
@@:
		test	ax,PF_PROMISCUOUS	;promiscuous mode?
		jz	@F			;no
		jmp	setmcregs		;yes
@@:		test	ServStat.ms_pktfilter,PF_PROMISCUOUS
						;were we before?
		jz	@F			;no
		;set in tables, we are promiscuous before
		mov	ServStat.ms_pktfilter,ax
		PUSH	AX			;reserver AX
		call	SetMCast		;yes. reset the multicast regs
		POP	AX			;restore AX
@@:		jmp	setexit

setmcregs:
		WRD_NIC  MAR0,0ffffffffh     ;set all bits in multicast array
		WRD_NIC  MAR4,0ffffffffh

setexit:	mov	ServStat.ms_pktfilter,ax  ;set in tables
		mov	ax,SUCCESS
		popf
		ret
L_ENDP		SetPacketFilter

COMMENT ^
-------------------------------------------------------------------------
$Name: FindAddr - Find an address in the multicast list$
-------------------------------------------------------------------------
Registers used:
	Input:
		ES:DI points to the address we are looking for.
	Output
		DS:SI points to address
		CX contains index of address in list (0 is the first element)

$Description:
This procedure searches for an address in the multicast address list. It
returns a pointer to the address in SI and the index of the address in cx.
CX contains -1 if the address is not found.
$

$Method:
No attempt is made to optimize this routine for speed, since it is rarely
used.
$
^
L_PROC		FindAddr
		mov	si,OFFSET MultiAddr.mt_mcast	;start of list
		mov	cx,MultiAddr.mt_curentry	;# of entries

		cmp	cx,0			;anything in list
		jne	@F			;entries not zero
		mov	cx,-1			;no. return not found
		ret

@@:		mov	ax, ES:[di]		;load address into regs
		mov	bx, ES:[di]+2		;second word
		mov	dx, ES:[di]+4		;third word

loopC:		cmp	ax,[si] 		;compare first word
		jne	loopend
		cmp	bx,[si]+2		;compare second word
		jne	loopend
		cmp	dx,[si]+4		;compare third word
		jne	loopend
		mov	ax,cx			;found. get index
		mov	cx,MultiAddr.mt_curentry;# of entries
		sub	cx,ax
		ret
loopend:	add	si,MAC_SNPA_SZ		;look at next entry
		loop	loopC

;address not found
		mov	cx,-1
		ret
L_ENDP		FindAddr

COMMENT %
-------------------------------------------------------------------------
$Name: SetMCast - Set the multicast list in the NIC registers$
-------------------------------------------------------------------------
$Description:
This procedure takes the multicast address list and sets the appropriate
bits in the NIC's MAR registers. The NIC filters multicasts using the CRC
of the multicast address; therefore, we must calculate the CRC of each
multicast address. The algorithm used is given below. It is derived from the
one given in the DP8390 Datasheet Addendum (Oct. 1988).

#define POLYNOMIAL 0x4c11db6

CalcCRC()
{
	unsigned long crc;
	int	i,j,k,carry,register,bitmask;

	crc = 0xffffffff;
	for (1=0; i<6; i++) {
		for (j=0; j<8; j++) {
			carry = ((crc & 0x80000000)?1:0) ^ (mult_addr[i] & 1);
			crc <<= 1;
			mult_addr[i] >>= 1;
			if (carry)
				crc = ((crc ^ POLYNOMIAL) | carry));
		}
	}
	crc = crc & 0xff000000;
	crc = crc >> 26;
	register = MAR0 + (crc >> 3);
	bitmask = 1 << (crc & 7);
	WR_NIC(register, bitmask);
}

REGISTER USAGE:
	si:di is used to store the CRC.
	dl is used to hold carry.
	dh is used to hold mult_addr[i].
	cx is used to hold loop counters
$

$Method:
$
%

L_PROC		SetMCast

		test	ServStat.ms_pktfilter,PF_PROMISCUOUS	;are we in prom. mode?
		jz	@F			;no
		ret				;yes. Don't mess with Mcast regs

@@:		pushf
		cli			;changing register page
		WRD_NIC  MAR0,0 	 ;clear multicast registers
		WRD_NIC  MAR4,0
		popf

		mov	cx, MultiAddr.mt_curentry	;number of addresses
		lea	bx, MultiAddr.mt_mcast		;first address
sm_loop1:	cmp	cx,0				;all addrs processed?
		jne	@F				;no
		jmp	sm_done 			;yes. We're done
@@:		push	cx				;save this count
		mov	si,0FFFFH			;initialize crc
		mov	di,0FFFFH			;in si:di
		mov	cx,6				;bytes in address
sm_loop2:	mov	dh, [bx]			;get address byte
		push	cx				;save this loop ctr
		mov	cx,8				;loop for CRC calc.
sm_loop3:	xor	dl,dl				;clear the carry byte
		shl	di,1				;shift left CRC
		rcl	si,1
		rcl	dl,1				;get high order bit of CRC
		mov	al,dh				;get low order bit
		and	al,1				;of addr byte
		xor	dl,al				;xor into carry
		shr	dh,1				;shift addr byte
		test	dl,1				;is carry set?
		jz	@F
		xor	si,04C1H			;xor CRC with
		xor	di,1DB6H			;0x4C11DB6
		or	di,1				;and set low order bit
@@:		loop	sm_loop3
		pop	cx				;get back byte count
		inc	bx				;go to next byte
		loop	sm_loop2

		and	si,0ff00h			;get high order bits of CRC
		mov	cl,10				;get high order 6 bits
		shr	si,cl
		mov	ax,si				;get CRC
		shr	ax,1				;get high order 3 bits
		shr	ax,1				;this is the multicast address
		shr	ax,1				;register to set the bit in.
		add	ax,MAR0 			;get the multicast register
		mov	cx,si				;get low order 3 bits of the CRC
		and	cx,7
		mov	di,1				;get bit to set
		shl	di,cl
		pushf
		cli
		mov	si,ax
		push	ax
		mov	al,cl
		xor	ah,ah
	      ; RD_NIC	AX, SI
		pop	ax
		MOV	AL, CL
		or	ax,di				;set bit
		push	ax
		xor	ah,ah
		WR_NIC	si,ax
		pop	ax
		popf
		add	bx, MAC_SNPA_SZ - 6		;go to next addr
		pop	cx				;get back count of addresses
		dec	cx
		cmp	cx,0				;are we done?
		jz	sm_done 			;yes
		jmp	sm_loop1			;no. another address
sm_done:	ret
L_ENDP		SetMCast

COMMENT ^
-------------------------------------------------------------------------
$Name: AddMulticastAddress - Add an address to the multicast list$
-------------------------------------------------------------------------
$Description:
This procedure adds a multicast address to the list. First it makes sure the
address is not already on the list. If not, it is added, and SetMCast is
called to set the bits in the NIC
$

$Method:
$
^
L_PROC		AddMulticastAddress
MAddr		EQU	<DWORD PTR [bp+10d]>

		pushf				  ;atomic access to table
		cli				  ;clear interrupt
		mov	bx,MultiAddr.mt_curentry  ;# of addresses now
		cmp	bx,MultiAddr.mt_mxentry   ;max #
		jne	@F			  ;are we already at max?
		popf				  ;pop flag
		mov	ax,INVALID_FUNCTION	  ;yes
		ret

@@:		push	es		;reserve es
		les	di,MAddr	;pointer to new address
		mov	al,ES:[di]	;is it a multicast address?
		and	al,1		;low order bit of high order byte
		jnz	@F		;it is multicast addr.
		pop	es		;not multicast addr.
		popf			;restore flag
		mov	ax,INVALID_PARAMETER	;no
		ret

@@:		call	FindAddr	;Is address in list?
		cmp	cx,-1
		je	@F		;not found
		pop	es		;found (duplicate multicast addr.)
		popf			;restor flag
		mov	ax,INVALID_PARAMETER	;no
		ret

;add it to the end of the list. Thanks to FindAddr, si points to the address
;where we should add it.
@@:		mov	ax, ES:[di]	;do the first word
		mov	[si],ax
		mov	ax, ES:[di]+2	;second word
		mov	[si]+2,ax
		mov	ax, ES:[di]+4	;third word
		mov	[si]+4,ax

;the rest of the buffer is not copied
		inc	MultiAddr.mt_curentry  ;increment number of entries
		pop	es		;restore es
		popf			;restore flag

		call	SetMCast	;set multicast bits
		mov	ax,SUCCESS
		ret
L_ENDP		AddMulticastAddress

COMMENT ^
----------------------------------------------------------------------------
$Name: DeleteMulticastAddress - remove a previously added multicast address$
----------------------------------------------------------------------------
$Description:
This procedure calls FindAddr to find the address in the list. It then copies
the last address in the list over the one we are deleting and decrements
the count. This keeps the list packed. Finally, we call SetMCast to set
the bits in the NIC.
$

$Method:
$
^
L_PROC		DeleteMulticastAddress

DAddr		EQU	<DWORD PTR [bp+10d]>

		pushf			     ;reserve flag
		cli			     ;clear interrupt
		push	es		     ;reserve es
		les	di, DAddr	     ;get the address to delete
		call	FindAddr	     ;find it in the list

		cmp	cx,-1		     ;was it found?
		jne	@F		     ;found
		pop	es		     ;no. error, not found
		popf			     ;restore flag
		mov	ax,INVALID_PARAMETER
		ret

;get offset of last entry in table
@@:		mov	di,MultiAddr.mt_curentry  ;# of entries
		dec	di	    ;get number of multicast addr. to skip
		shl	di,1	    ;multiply by 16 (entry size)
		shl	di,1
		shl	di,1
		shl	di,1

;do the move. FindAddr has set si to point to the entry we are deleting.
		mov	ax, WORD PTR MultiAddr.mt_mcast[di]
		mov	[si],ax
		mov	ax, WORD PTR MultiAddr.mt_mcast[di]+2
		mov	[si]+2,ax
		mov	ax, WORD PTR MultiAddr.mt_mcast[di]+4
		mov	[si]+4,ax

		dec	MultiAddr.mt_curentry
				    ;decrease current multicast addr. number
		pop	es	    ;restore es
		popf		    ;restore flag

		call	SetMCast    ;set multicast addr. bits
		mov	ax, SUCCESS
		ret
L_ENDP		DeleteMulticastAddress

COMMENT ^
-------------------------------------------------------------------------
$Name: ReadCounters - Read the 8390 statistics counters
-------------------------------------------------------------------------
;hold 95.12.18 in int.asm

$Description:
This routine reads and clears the statistics counters on the 8390. The
counters are cleared by reading them.
$

$Method:
$
^
A_PROC	ReadCounters
		cli				;atomic operation
		xor	ax,ax			;clear ax
		sti
		ret
		MOV	AL, BL
		DWADD	MedStat.me_alerr,ax	;update stat
		MOV	AL, BL
		cmp	al,0			;any of them?
		je	@F			;no
		DWADD	MacStat.ms_RxCrcPktCT,ax	;update stats
		or	MedStat.me_rcvmsk._LO,RCVCRCERR ;set bit in mask
@@:
		MOV	AL, BL
		DWADD	MacStat.ms_RxNoBufPktCT,ax	;update stats
		sti
		ret
A_ENDP	ReadCounters

COMMENT ^
-------------------------------------------------------------------------
$Name: UpdateStatistics - Make sure stats are up to date$
-------------------------------------------------------------------------
;hold 95.12.18 in INT.asm

$Description:
This reads the counters on the 8390. All other stats are current.
$

$Method:
$
^
L_PROC		UpdateStatistics
		call	ReadCounters
		mov	ax,SUCCESS
		ret
L_ENDP		UpdateStatistics

COMMENT ^
-------------------------------------------------------------------------
$Name: ClearStatistics - Initialize the statistics tables$
-------------------------------------------------------------------------
$Description:
This procedure clears the statistics kept by the MAC.
$

$Method:
$
^
L_PROC		ClearStatistics

		cli				   ;do this atomically

IFNDEF		DOS				   ;DOS
		mov	es, SysInfoSeg		   ;get time of clear
		mov	ax, ES: SIS_BigTime._lo    ;and write it
		mov	ServStat.ms_dt_clrstats._lo,ax
		mov	ax, ES: SIS_BigTime._hi
		mov	ServStat.ms_dt_clrstats._hi,ax
ENDIF
		mov	si, OFFSET MacStat		;MAC stats table
		mov	[si].ms_RxTotPktCT._LO,0	;total packets received
		mov	[si].ms_RxTotPktCT._HI,0
		mov	[si].ms_RxCRCPktCT._LO,0	;CRC errors
		mov	[si].ms_RxCRCPktCT._HI,0
		mov	[si].ms_RxTotByteCT._LO,0	;Bytes Received
		mov	[si].ms_RxTotByteCT._HI,0
		mov	[si].ms_RxNoBufPktCT._LO,0	;Packets discarded
		mov	[si].ms_RxNoBufPktCT._HI,0
		mov	[si].ms_TxTotPktCT._LO,0	;packets transmitted
		mov	[si].ms_TxTotPktCT._HI,0
		mov	[si].ms_TxHwerrPktCT._LO,0	;Hardware errors
		mov	[si].ms_TxHwerrPktCT._HI,0

		mov	si, OFFSET MedStat		;media statistics table
		mov	[si].me_alerr._LO,0		;alignment errors
		mov	[si].me_alerr._HI,0
		mov	[si].me_16coll._LO,0		;16 collision packets
		mov	[si].me_16coll._HI,0
		mov	[si].me_rcvmsk._LO,0		;receive error mask
		mov	[si].me_rcvmsk._HI,0
		mov	[si].me_xmtmask._LO,0		;transmit error mask
		mov	[si].me_xmtmask._HI,0

		sti					;restore interrupts

		mov	ax,SUCCESS
		ret
L_ENDP		ClearStatistics

COMMENT ^
-------------------------------------------------------------------------
$Name: SetLookAhead - handle setlookahead request
-------------------------------------------------------------------------
$Description:
Our lookahead is always at least 252 bytes, because of the way the 8390
works (256 byte page - 4 status bytes). If they ask for more, we say no.
$

$Method:
$
^
L_PROC		SetLookAhead
Lookahead	EQU		<WORD PTR [bp+14d]>

		cmp	Lookahead, 1600d	;more than we support?
		ja	@F			;yes
		MOV	AX, LookAhead
		CMP	AX, LookAheadValue
		JBE	NoChange
		MOV	LookAheadValue,AX
NoChange:	mov	ax,SUCCESS		;no. Call ok.
		ret
@@:		mov	ax, INVALID_PARAMETER	;Too much requested
		ret
L_ENDP		SetLookAhead

COMMENT ^
-------------------------------------------------------------------------
$Name: Request - vector general requests to the appropriate procedures$
-------------------------------------------------------------------------
Calling Procedure:
	PUSH	WORD	ProtID		;Module ID of protocol
	PUSH	WORD	ReqHandle	;handle of request
	PUSH	WORD	Param1		;Request dependent word parameter
	PUSH	DWORD	Param2		;Request dependent dword parameter
	PUSH	WORD	Opcode		;Opcode of request
	PUSH	WORD	MacDS		;DS of called MAC module
	CALL	Request

$Description:
This procedure saves all the registers, sets up the stack, and vectors
to the appropriate routine for the request. Upon return from the routine,
the return value is stored, the registers are restored and the return value
is put in  AX.
$

$Method:
$
^

A_PROC		Request, FAR
		push	bp			;set up stack frame
		mov	bp,sp
		push	ds			;save registers
		push	di
		push	si
		mov	ds,MacDS		;get our ds
		mov	bx, Opcode
		cmp	bx,REQ_RUN_DIAGNOSTICS	;bounds check
		jl	invalid
		cmp	bx,REQ_SET_LOOKAHEAD
		jg	invalid

		sub	bx,1			;get index into dispatch table
		shl	bx,1			;word lengths
		call	dispatch[bx]		;call appropriate procedure

		jmp	clean

invalid:	mov	ax, INVALID_FUNCTION	;return for invalid opcodes


clean:
		pop	si			;restore registers
		pop	di
		pop	ds
		pop	bp
		ret	14d
A_ENDP		Request

;============================================================================
L_PROC		DelayTest
		PUSH   CX
		MOV    CX, 10D
@@:		JMP    $+2
		LOOP   @B
		POP    CX
		ret
L_ENDP		DelayTest
;============================================================================
L_PROC		DelayTest1
		PUSH   CX
		MOV    CX, 2D
@@:		JMP    $+2
		LOOP   @B
		POP    CX
		ret
L_ENDP		DelayTest1


CODESEG_END


INITCODESEG_BEG

COMMENT ^
----------------------------------------------------------------------------
$Name: Init_Request - Initialize the request module
----------------------------------------------------------------------------
$Description:
This module does nothing now.
$

$Method:
$
^

A_PROC	    Init_Request

IFNDEF		DOS			;this stuff is not relevant in DOS

					;OS/2
		;************************ <OS/2> **************************
		push	es
;get SysInfoSeg GDT Selector
		mov	al,DOSVar_SysInfo
		mov	dl,DEVHLP_GETDOSVAR
		call	[Device_Help]
		jnc	@F
		mov	ax,1
		pop	es
		ret

@@:	       mov     es,ax
	       mov     ax,WORD PTR es:[bx]
	       mov     SysInfoSeg, ax
	       pop     es
		;************************ <OS/2> **************************
ENDIF
		mov	ax, SUCCESS
		ret
A_ENDP	Init_Request

INITCODESEG_END
END
