COMMENT  ^

Contains the strategy routine for the driver.  This handles init and
the generic IOCTL mechanism for getting the address of the common
characteristics table.

^

INCLUDE macros.inc
INCLUDE os2defs.inc
INCLUDE dp8139.inc
INCLUDE RTL8139.inc

;--------------------------------------------------------------------------
MACcategory     EQU     82h             ;category for generic IOCTL
MACfunction     EQU     41h             ;function for generic IOCTL

;==========================================================================
EXTRN   CommChar:WORD
EXTRN   Init:NEAR
EXTRN   IOBase:WORD
DATASEG_BEG
InitDone        dw      0               ;flag = 1, if init is done
DATASEG_END
;==========================================================================
CODESEG_BEG


;--------------------------------------------------------------------------

COMMENT ^
----------------------------------------------------------------------------
$Name: strategy - strategy routine for the driver$
----------------------------------------------------------------------------
$Description:
This is the strategy entry for the driver. It handles both the init command
and the ioctl command used to get the address of the common characteristics
table.  If init is called more than once, an error is returned (the init
code is dumped after config.sys time).
$

$Method:
$
^
;--------------------------------------------------------------------------
;                      PROCEDURE NAME --- Strategy
;--------------------------------------------------------------------------
A_PROC          Strategy, FAR        ;A far procedure
;               cli
IFDEF DOS                            ;DOS
                push    ax           ;push register
                push    bx
                push    cx
                push    dx
                push    bp
                push    es
                pushf
ENDIF
IFDEF   DOS                          ;DOS
                push    cs           ;set ds is equal to cs
                pop     ds
ENDIF
                mov     al, es:[bx].PKTCMD   ;packet command
                cmp     al,CMDInit   ;CMDInit=0 (initial)
                jne     err          ;not for initial

;see if Init has already been called. If it has, don't allow it to be
;called again. (0 for not done, 1 for done)
                cmp     InitDone,0              ;Init already run?
                jne     err                     ;Init has done
                call    Init                    ;init the driver
                mov     InitDone,1              ;flag that init has run

                jmp     done

err:            mov     ax, STDON+STERR+ERROR_I24_BAD_COMMAND
                                                ;error=bad command
                mov     es:[bx].PKTSTATUS,ax    ;return error code

done:

IFDEF   DOS                                     ;DOS
                popf
                pop     es
                pop     bp
                pop     dx
                pop     cx
                pop     bx
                pop     ax
ENDIF
;               sti
                ret

A_ENDP          Strategy

;--------------------------------------------------------------------------
;                     PROCEDURE NAME --- DosInt
;--------------------------------------------------------------------------
;DOS calls this routine after strategy is called. Really, strategy isn't
;supposed to do anything but store the message and this routine should
;do it all. Tough break for DOS, though, as in this driver strategy does
;everything and Interrupt does nothing

A_PROC          DosInt, FAR
                xor   ax,ax
                ret
A_ENDP          DosInt
;--------------------------------------------------------------------------


CODESEG_END
;==========================================================================
INITCODESEG_BEG
COMMENT ^
----------------------------------------------------------------------------
$Name: Init_Strategy - Initialize the Strategy module
----------------------------------------------------------------------------
$Description:
This routine initializes the Strategy module. It does nothing currently.
$

$Method:
$
^
;--------------------------------------------------------------------------
;                  PROCEDURE NAME --- Init_Strategy
;--------------------------------------------------------------------------
A_PROC          Init_Strategy
                mov     ax,SUCCESS
                ret
A_ENDP          Init_Strategy
INITCODESEG_END
;==========================================================================

END


