COMMENT  ^
This module contains the device header and MAC driver tables.
^
;includ files

INCLUDE macros.inc
INCLUDE vector.inc
INCLUDE os2defs.inc
INCLUDE dp8139.inc
INCLUDE RTL8139.inc
INCLUDE devhdr.inc
;==============================================================================
;==============================================================================
EXTRN   BrdNum : BYTE
EXTRN   MultiAddr:WORD                  ;ref. request.asm, vector.inc
EXTRN   strategy:FAR , System:NEAR, Request:FAR , TransmitChain:FAR
EXTRN   TransferData:FAR , ReceiveRelease:FAR , IndicationOn:FAR
EXTRN   IndicationOff:FAR , IOBase:WORD
EXTRN   DosInt:FAR

DATASEG_BEG


; Device driver header

;------------------------------------------------------------------------------
;                      PUBLIC NAME --- Driver Head
;------------------------------------------------------------------------------
PUBLIC DriverHead
PUBLIC DriverName
DriverHead LABEL WORD
        dw      -1,-1                   ;Header links
IFDEF   DOS                             ;DOS
        dw      DEV_CHAR_DEV+DEV_SUP_IOCTL+DEV_GIOCTL
                ;char dev driver with IOCTL
                ;DEV_CHAR_DEV:a char device (ref. devhdr.inc)
                ;DEV_GIOCTL:device support generic ioctl
                ;DEV_SUP_IOCTL:device supports IOCTL (DOS)
        dw      offset DGROUP:strategy  ;offset of strategy routine
        dw      offset DGROUP:DosInt    ;offset of DosInt routine
ELSE                                    ;OS/2
        ;************************ <OS/2> **************************
        dw      DEV_CHAR_DEV+DEVLEV_1+DEV_GIOCTL
        dw      offset strategy         ;offset of strategy routine
        dw      0                       ;reserved
        ;************************ <OS/2> **************************
ENDIF
DriverName       db      'RTSND'                  ;driver name (must be 7
                 db      '$'
                 db      ' '
                 db      ' '
                 db      '$'

bnum1            db      '$'
                 db      ' '                     ;$ terminates driver name
                 db      ' '

                 dw      0,0,0,0                 ;reserved
                 dd      40h
;------------------------------------------------------------------------------
;                      PUBLIC NAME --- CommChar
;------------------------------------------------------------------------------
; Common characteristics table. A value of -1 indicates that this is
; filled in at init time by the driver.

PUBLIC  CommChar
CommChar        LABEL   WORD
                dw      SIZE com_char          ;table size
                dw      0                      ;level of table
                dw      0                      ;level of subtables
                db      MAJORVERS              ;major module version
                db      MINORVERS              ;minor module version
                dd      FUNC_FLAG_UPPER_BIND   ;module function flags (1)
ModNm           db      MODULENAME             ;module name
bnum2           db      0                      ;board number
                db      0                      ;terminate string
                org     ModNm + VEC_MODNAME_SZ ;module name is 16 bytes
                db      MAC_PROTO_LEVEL        ;upper boundary protocol (MAC)
                                               ;MAC_PROTO_LEVEL=1
                db      MAC_MAC_INTF           ;upper boundary interface (MAC)
                                               ;MAC_MAC_INTF=1
                db      PHYS_PROTO_LEVEL       ;lower boundary protocol
                                               ;PHYS_PROTO_LEVEL=0
                db      MAC_MAC_INTF           ;lower boundary interface (MAC)
                                               ;MAC_MAC_INTF=1
                dw      -1                     ;module id (filled in later)
                dw      -1                     ;module ds (filled in later)
IFDEF DOS                                      ;DOS
                dw      offset DGROUP:System   ;ptr to system request entry
ELSE                                           ;OS/2
                ;************************ <OS/2> **************************
                dw      offset CGROUP:System   ;ptr to system request entry
                ;************************ <OS/2> **************************
ENDIF
                dw      ?                      ;store segment(filled in later)
                dw      offset ServChar        ;pointer to service-specific
                                               ;characteristics
                dw      ?                      ;store segment(filled in later)
                dw      offset ServStat        ;pointer to service statistics
                dw      ?                      ;store segment(filled in later)
                dw      offset UDisp           ;pointer to upper dispatch table
                dw      ?                      ;store segment(filled in later)
                dd      NULL                   ;pointer to lower dispatch table
                dd      NULL,NULL              ;reserved
;------------------------------------------------------------------------------
;                       PUBLIC NAME --- ServChar
;------------------------------------------------------------------------------
;service specific characteristics table. A value of -1 indicates that this
;is filled at at init time by the driver

PUBLIC          ServChar

ServChar        dw      SIZE mac_ss_char        ;length of table
TName           db      "DIX+802.3",0           ;type of MAC
                org     TName + MAC_SRVTYPE_SZ  ;field is 16 bytes long
                dw      6                       ;bytes of ethernet address
                db      MAC_SNPA_SZ dup(-1)     ;permanent station address
                                                ;MAC_SNPA_SZ=16
                db      MAC_SNPA_SZ dup(-1)     ;current station address
                dd      0                       ;adaptor functional address (none)
                                                ;(0 if none)
                dw      offset MultiAddr        ;multicast address list
                dw      ?                       ;store segment(filled in later)
                dd      10000000d               ;link speed (10Mbits/sec)
                dd      SERVFL
                        ;service flags = Broadcast+Multicast+Promiscuous
                        ;                +Settable station addr.
                        ;                +GDT virtual addr.
                dw      MAXPACKET               ;maximum frame size (1514)
                dd      6*256*2                 ;xmit buffer capacity 6*256*2
                dw      2048D                   ;xmit buffer block size (1536)
                dd      0                       ;receive buffer capacity
                dw      256D                    ;receive buffer block size(
                db      00h,0C0h,0F6H            ;IEEE vendor code
                db      0                       ;vendor adaptor code
                dd      AdaptDesc               ;pointer to adaptor description

AdaptDesc       db      'Fast Ethernet ',0


;------------------------------------------------------------------------------
;                       PROCEUDRE NAME --- Udisp
;------------------------------------------------------------------------------
;MAC upper dispatch table (addresses for the following functions)
;1. CommChar          2. Request           3. TransmitChain
;4. TransferData      5. ReceiveRelease    6. IndicationOn
;7. IndicationOff

PUBLIC  Udisp

IFDEF DOS                                  ;DOS
UDisp    dw   offset DGROUP:CommChar       ;back pointer to common char table
         dw   ?                            ;store segment(filled in later)
         dw   offset DGROUP:Request        ;request address
         dw   ?                            ;store segment(filled in later)
         dw   offset DGROUP:TransmitChain  ;TransmitChain address
         dw   ?                            ;store segment(filled in later)
         dw   offset DGROUP:TransferData   ;TransferData address
         dw   ?                            ;store segment(filled in later)
         dw   offset DGROUP:ReceiveRelease ;ReceiveRelease Address
         dw   ?                            ;store segment(filled in later)
         dw   offset DGROUP:IndicationOn   ;IndicationOn address
         dw   ?                            ;store segment(filled in later)
         dw   offset DGROUP:IndicationOff  ;IndicationOff address
         dw   ?                            ;store segment(filled in later)
ELSE                                       ;OS/2
         ;************************ <OS/2> **************************
UDisp    dw   offset DGROUP:CommChar       ;back pointer to common char table
         dw   ?                            ;store segment(filled in later)
         dw   offset CGROUP:Request        ;request address
         dw   ?                            ;store segment(filled in later)
         dw   offset CGROUP:TransmitChain  ;TransmitChain address
         dw   ?                            ;store segment(filled in later)
         dw   offset CGROUP:TransferData   ;TransferData address
         dw   ?                            ;store segment(filled in later)
         dw   offset CGROUP:ReceiveRelease ;ReceiveRelease Address
         dw   ?                            ;store segment(filled in later)
         dw   offset CGROUP:IndicationOn   ;IndicationOn address
         dw   ?                            ;store segment(filled in later)
         dw   offset CGROUP:IndicationOff  ;IndicationOff address
         dw   ?                            ;store segment(filled in later)
         ;************************ <OS/2> **************************
ENDIF
;------------------------------------------------------------------------------
;                       PUBLIC NAME --- ServStat
;------------------------------------------------------------------------------
;service specific status table. A value of -1 means this statistic is not
;kept. These are the tables that are readable by the protocol. For the
;sake of consistency, the actual statistics are kept in separate tables
;and copied here when necessary

PUBLIC          ServStat

ServStat        dw      SIZE mac_ss_status      ;length of status table
                dd      -1                      ;date/time of last diagnostics
                dd      0                       ;mac status
                dw      0
                ;current packet filter
                ;0 - directed and multicast or group and functional
                ;1 - broadcast
                ;2 - promiscuous
                ;3 - all source routing
                ;4 - 15 - reserved, must be zero
                dw      offset MedStat          ;pointer to media specific
                                                ;stat table
                dw      ?                       ;store segment(filled in later)
                dd      -1                      ;time when statistics cleared
;------------------------------------------------------------------------------
;                         PUBLIC NAME --- MacStat
;------------------------------------------------------------------------------
;MAC driver statistics

PUBLIC MacStat
MacStat         dd      0               ;total frames received
                dd      0               ;frames with crc error
                dd      0               ;total bytes received
                dd      0               ;frames discarded - no buffer space
                dd      -1              ;multicast frames received
                dd      -1              ;broadcast frames received
                dd      -1              ;frames received with errors
                dd      -1              ;frames exceeding maximum size
                dd      -1              ;frames smaller than minimum size
                dd      -1              ;multicast bytes received
                dd      -1              ;broadcast bytes received
                dd      -1              ;frames received - hardware error
                dd      0               ;total frames transmitted
                dd      -1              ;total bytes transmitted
                dd      -1              ;multicast frames transmitted
                dd      -1              ;broadcast frames transmitted
                dd      -1              ;broadcast bytes transmitted
                dd      -1              ;multicast bytes transmitted
                dd      0               ;frames not transmitted - time out
                dd      0               ;frames not transmitted - HW error
;------------------------------------------------------------------------------
;                        PUBLIC NAME --- MedStat
;------------------------------------------------------------------------------
;media specific status table. A value of -1 indicates that the statistic
;is not kept

PUBLIC  MedStat
MedStat         dw      SIZE mac_eth_stats      ;length of table
                dw      1               ;802.3 statistics structure version
                dd      0               ;frames with alignment error
                dd      0               ;receive error failure mask
                dd      -1              ;frames with overrun error
                dd      -1              ;frames sent with at least 1 collision
                dd      -1              ;frames sent after deferring
                dd      0               ;frames not sent - 16 collisions
                dd      -1              ;total # of collisions while xmitting
                dd      -1              ;number of late collisions
                dd      -1              ;frames xmitted after 1 collision
                dd      -1              ;frames Xmitted after multiple colls.
                dd      -1              ;frames xmitted, CD heartbeat
                dd      -1              ;Jabber errors
                dd      -1              ;Carrier senses lost during Xmit
                dd      0               ;Transmit error mask
DATASEG_END
;==============================================================================

;==============================================================================
INITCODESEG_BEG

COMMENT ^


--------------------------------------------------------------------------------
$Name: Init_Tables - Initialize the Tables module (filling card of the parameter)
--------------------------------------------------------------------------------
$Description:
This routine initializes the Tables module. It patches the long pointers
and DS values in the tables to use the protected mode CS and DS.
It also fills the board number in to the device header.
$

$Method:
$
^
;------------------------------------------------------------------------------
;        PROCEDURE NAME --- Init_Tables (To check how many bords are there ?)
;------------------------------------------------------------------------------
A_PROC          Init_Tables

                LEA     si,Drivername
                mov     cx,10h
Search_A:
                cmp     byte ptr [si] ,'$'
                je      @F
                inc     si
                loop    Search_A
@@:

                MOV     AL, BrdNum
                ADD     AL, '0'
                CMP     Brdnum,1                ;default=1
                JA      TowBoard
                MOV     BYTE PTR [si], '$'
                MOV     BYTE PTR [si+1], '$'   ;end signature
                MOV     BYTE PTR bnum2, AL
                jmp     @F
TowBoard:

                MOV     AL, BrdNum
                ADD     AL, '0'
                MOV     BYTE PTR [si], AL
                MOV     BYTE PTR [si+1], '$'   ;end signature
                MOV     BYTE PTR bnum2, AL
@@:



IFDEF           DOS                             ;DOS
;in DOS, the device header does not include the protected mode selectors.
;Hence, we set them here so the code works the same.
                mov     ax,ds                   ;get data segment
                mov     DriverHead.SProtDS,ax   ;protected mode DS
                mov     DriverHead.SRealDS,ax   ;real mode DS
                mov     ax,cs                   ;get code segment
                mov     DriverHead.SProtCS,ax   ;protected mode CS
                mov     DriverHead.SRealCS,ax   ;real mode CS
ENDIF
;pointers to code segment
                mov     ax, DriverHead.SProtCS  ;get protected mode CS
                mov     CommChar.cc_sysreq._seg,ax
                mov     UDisp.mu_request._seg, ax
                mov     UDisp.mu_xmtchain._seg, ax
                mov     UDisp.mu_xfrdata._seg, ax
                mov     UDisp.mu_rxbuffree._seg, ax
                mov     UDisp.mu_indon._seg, ax
                mov     UDisp.mu_indoff._seg, ax

;pointers to data segment
                mov     ax, DriverHead.SProtDS  ;get protected mode DS
                mov     CommChar.cc_dataseg,ax
                mov     CommChar.cc_un_char._seg,ax
                mov     CommChar.mac_charpt._seg,ax
                mov     CommChar.cc_un_udt._seg,ax
                mov     ServChar.mc_mcast_pt._seg,ax
                mov     ServChar.mc_serialnum._seg,ax
                mov     ServStat.ms_mediastatspt._seg,ax
                mov     Udisp.mu_comcharpt._seg,ax

;board characteristics
                mov     ServChar.mc_boardcode,0         ;and put in tables

                sti                                     ;set interrupt
                mov     ax,TxbufferSize                 ;Tx pages 4
                mov     ServChar.mc_xmtbufsz._lo,ax     ;write size to tables
                mov     ax,64                           ;rx buffer length
                mov     ServChar.mc_rcvbufsz._lo,ax     ;write size to tables

                mov     ax,SUCCESS                      ;success
                ret
A_ENDP          Init_Tables

INITCODESEG_END
;==============================================================================

END

