COMMENT  ^
This module is responsible for packet transmission. It contains routines
to deal with a transmit request from the host as well as a completed
transmission.
^
INCLUDE macros.inc
INCLUDE vector.inc
INCLUDE os2defs.inc
INCLUDE dp8139.inc
INCLUDE RTL8139.inc
include bit.inc

IFNDEF  DOS                             ;286 protected mode needed to
.286P                                   ;examine machine status word
ENDIF                                   ;under OS/2

;TRANSMIT CONTROL STRUCTURE
;These are used to track the transmit buffers on the board. Each buffer
;is statically assigned a TCS. All TX buffers are large enough to hold a
;maximally sized packet.  The driver uses the size and free fields to
;determine the status of the buffer. If the size is greater than 0, the
;buffer is waiting to be transmitted. If the free field is 1, the buffer
;is available to be filled. If the free field is 0, the buffer is not
;available.



;==============================================================================
DATASEG_BEG

EXTRN   CommChar:WORD, ProtDS:WORD, MacStat:WORD, MedStat:WORD
EXTRN   FOR_RET_FLAG:WORD,Sub_vender_ID:byte
EXTRN   NICCR9346:word,NICCONFIG0:WORD,NICCR9346:WORD,TxStartAddrPage0:byte
EXTRN   NICCMD:word,NICTCR:word,NICIMR:word,NICRCR:word
EXTRN   CurRxBufAddr:word,RxBufBoundary:word,RxBufferStartAddr:byte
EXTRN   NICGEP:word,NICCONFIG1:word,NICMAR0:word,NICMAR4:word
EXTRN   AutoDetected:Near
EXTRN   RxBurstDMA:word,TxBurstDMA:WORD,NICIDR0:word
EXTRN   RESV1:dword,RESV2:dword,RESV3:dword


Tx_Cont_Block   Tx_Control  8 dup(<>)   ;Tx control blocks
Public Tx_Cont_Block
PUBLIC TCSNextToFill, TCSNextToXmit, TCSNextToTOK
TCSNextToFill   dw      ?       ;pointer to next TX_Control to be filled
TCSNextToXmit   dw      ?       ;pointer to next Tx_Control to be xmitted
TCSNextToTOK    dw      ?       ;pointer to next Tx_Control to be xmitted OK
XmitInProg      dw      0       ;1 if transmit in progress, 0 means done
Prog_Xmit       dw      0
XmitStartTime   dw      ?       ;start of Xmission in Ticks
TickCount       dw      0       ;current clock ticks
Active_count    dw      0
;;;3/28/98
Public  Txcount, XmitCount
TxCount         db      0
XmitCount       db      0
debug1          db      0


;;;3/28/98
IFDEF   DOS
NextTimer       dd      ?       ;pointer to next timer routine
Count_link      db      0
test_down       db      0
ENDIF


PUBLIC  TransmitConfirm
TransmitConfirm dd      ?       ;address of protocol's Transmit_confirm routine

EXTRN   Device_Help:DWORD
extrn   segTX:WORD
DATASEG_END
;==============================================================================
CODESEG_BEG
EXTRN   cs_ds:WORD
EXTRN   IOBase:WORD
EXTRN   NICTSD0:WORD,NICTSD1:WORD,NICTSD2:WORD,NICTSD3:WORD
EXTRN   NICTSAD0:WORD,NICTSAD1:WORD,NICTSAD2:WORD,NICTSAD3:WORD
EXTRN   NICISR:WORD,NICTCR:WORD,NICTSAD:WORD,NICIMR:WORD
EXTRN   ResetNIC:near,wait_pci:near
;extrn   show_char:near,show_char1:near,show_char2:near
;extrn   disp_hex2:near,disp_hex4:near

;inserted on 2006/02/16 to avoid out of resource	begin
OWN_BIT0	equ	0001h
OWN_BIT1	equ	0002h
OWN_BIT2	equ	0004h
OWN_BIT3	equ	0008h
;inserted on 2006/02/16 to avoid out of resource	end


Vidseg		dw	0b800h				;Video Display Segment (0b000h = monochrome, 0b800h = color)
curpos  	dw      BEG_OF_SCR
BEG_OF_SCR      equ     0400h
END_OF_SCR      equ     0e60h


	public pchar
pchar   proc    near
        push    es
        push    bx

        push    ax
	
        mov     ax,cs:[Vidseg]             ;VGA ---> b800h, MGA ---> b000h
        mov     es,ax
        pop     ax
        mov     bx,cs:curpos
        mov     es:[bx],al
        mov     es:[bx+1],bh
        inc     bx
        inc     bx
        ;mov     byte ptr es:[bx],' '
        mov     byte ptr es:[bx],21h
        mov     byte ptr es:[bx+1],0aeh
        cmp     bx,END_OF_SCR
        jb      PC_10
        mov     bx,BEG_OF_SCR
PC_10:
        mov     cs:curpos,bx
        pop     bx
        pop     es
        ret
pchar   endp



	public pbyte
pbyte   proc     near
        push     ax
        push     ax
        shr      al,1
        shr      al,1
        shr      al,1
        shr      al,1
        cmp      al,0ah
        jb       PB_1
        add      al,'A'-'0'- 10
PB_1:
        add      al,'0'
        call     pchar
        pop      ax
        and      al,0fh  
        cmp      al,0ah
        jb       PB_2
        add      al,'A'-'0'- 10
PB_2:
        add      al,'0'
        call     pchar
        pop      ax
        ret
pbyte          endp



COMMENT ^
----------------------------------------------------------------------------
$Name: Start_Xmit - start transmission on the board, if possible.
----------------------------------------------------------------------------
$Description:
This procedure starts transmission on the board if possible. It is possible
if the Transmit Control Structure pointed to by TCSNextToXmit has a size
greater than zero. If we start transmission, we zero the size so we don't
transmit it again.
$
^
PUBLIC  Start_Xmit_1
L_PROC  Start_Xmit
        CLI                             ;critical section
        push    eax

        cmp     XmitCount, 4
        je      sx_done
        
;inserted on 2006/02/16 to avoid out of resource	begin
	pushf
	push	ax
	push	cx
count_own_bit:
	xor	cx,cx
	mov	dx,DS:NICTSAD
	in	ax,dx
	test	ax,OWN_BIT0
	jz	do_not_count0
	inc	cx
do_not_count0:
	test	ax,OWN_BIT1
	jz	do_not_count1
	inc	cx
do_not_count1:
	test	ax,OWN_BIT2
	jz	do_not_count2
	inc	cx
do_not_count2:
	test	ax,OWN_BIT3
	jz	do_not_count3
	inc	cx
do_not_count3:
	cmp	cx,4
	jb	count_own_bit
tx_continue:
	pop	cx
	pop	ax
	popf
;inserted on 2006/02/16 to avoid out of resource	end

        xor     eax, eax
        mov     bx,TCSNextToXmit        ;Next TCS to send
        mov     ax,[bx].TCS_Size        ;get size of packet
        cmp     ax,0                    ;any data to send?
        je      sx_done                 ;none to transmit
Start_Xmit_1:
        pushf
  ;      cli                             ;Critical section

        push    eax                      ;save ax register
        push    cx                      ;save transmit byte count
        mov     al,[bx].TCS_Page        ;Get start page of transmit buffer
        and     al, 3
        cmp     al,0                    ;check page 0
        je      p0                      ;yes  page 0
        cmp     al,1                    ;check page 1
        je      p1                      ;yes  page 1
        cmp     al,2                    ;check page 2
        je      p2                      ;yes  page 2
        cmp     al,3                    ;check page 3
        je      p3                      ;yes  page 3
        jmp     pend                    ;jmp end

 p0:
        mov     dx, DS:NICTSAD0
        mov     eax, [bx].TCS_PAddr
        out     dx, eax
        mov     dx,ds:NICTSD0

       jmp      pend

 p1:
        mov     dx, DS:NICTSAD1
        mov     eax, [bx].TCS_PAddr
        out     dx, eax
        mov     dx,ds:NICTSD1
        jmp      pend

 p2:
        mov     dx, DS:NICTSAD2
        mov     eax, [bx].TCS_PAddr
        out     dx, eax
        mov     dx,ds:NICTSD2
       jmp      pend

 p3:
        mov     dx, DS:NICTSAD3
        mov     eax, [bx].TCS_PAddr
        out     dx, eax
        mov     dx,ds:NICTSD3

 pend:
        pop     cx                      ;restore transmit byte count
        pop     eax                     ;restore ax

        add     eax, RESV1
        out     dx,eax                  ;write transmit byte count
        mov     [bx].TCS_Size,0         ;Zero packet size
        mov     XmitInProg,1            ;Flag transmit in progress
     ;3/28/98
        inc     XmitCount
        dec     TxCount
        mov     ax,TickCount            ;Get current time
        mov     XmitStartTime,ax        ;And save it

        mov     ax,[bx].TCS_Link;update pointer to next Tx_control
        mov     TCSNextToXmit,ax;refresh next TCS to fill

        popf


sx_done:
       ;   STI

@@:
        pop     eax
        ret
L_ENDP  Start_Xmit

COMMENT ^
----------------------------------------------------------------------------
$Name: Xmit_Done - handle a completed transmission
----------------------------------------------------------------------------
$Description:
This procedure is called when a packet transmission completes. It starts
the next transmission, if any and calls Transmit_Confirm
$
^
public Xmit_Done

A_PROC  Xmit_Done ,near
          CLI

        pushf                           ;reserve flags
        push    es                      ;reserve es,si,di
        push    si
        push    di

;inserted on 2006/02/16 to debug	begin
;	pushf
;	push	ax
;	mov	al,'X'
;	call	pchar
;	pop	ax
;	popf
;inserted on 2006/02/16 to debug	end

again:
;3/28/98
        ;cmp    XmitCount,0
        ;jz     xd_exit
        mov     ax, TCSNextToTOK
        cmp     TCSNextToXmit, ax
        jz      xd_exit


        mov     si,TCSNextToTOK        ;get TCS of completed transmission
        mov     al,[si].TCS_Page        ;get transmit page 0 - 3
        and     al, 3
        mov     dx,ds:nictsad
        cmp     al,0                    ;check page 0
        je      page0                   ;yes
        cmp     al,1                    ;check page 1
        je      page1                   ;yes
        cmp     al,2                    ;check page 2
        je      page2                   ;yes
        jmp     page3                   ;check page 3
                                        ;yes
page0:
        in      ax,dx
        and     ax,1111h               ;keep Descriptor 0 value
        jmp     page_end
page1:
        in      ax,dx
        and     ax,2222h               ;keep Descriptor 1 value
        jmp     page_end
page2:
        in      ax,dx
        and     ax,4444h               ;keep Descriptor 2 value
        jmp     page_end
page3:
        in      ax,dx
        and     ax,8888h               ;keep Descriptor 3 value
        jmp     page_end
page_end:
        test    ax,0f0f0h               ;did transmission succeed?
        jnz     @F                      ;yes.
        jmp     xd_exit
@@:

        test    ax,0f000h
        jnz     Xmit_OK

        mov     dx,DS:NICISR
        mov     al,ISR_TER
        out     dx,al

        mov     dx,DS:NICTCR
        mov     al,01
        out     dx,al                   ;Clear Abort


        jmp     xd_exit
Xmit_OK:

        MOV     XmitInProg,0            ;xmit no longer in progress
        ;3/28/98
        dec     XmitCount

        mov     [si].TCS_Free, TCS_AVAIL;structure is free (available)
        mov     ax,[si].TCS_Link        ;put next packet to transmit into ax
        mov     TCSNextToTOK,ax        ;set the pointer
        mov     ax,[si].TCS_Handle      ;get handle of TX request
        cmp     ax,0                    ;was it 0?
        je      xd_done                  ;yes. no confirmation necessary
@@:     PUSH    di
        mov     [si].TCS_Handle,0       ;make sure don't confirm twice
        ;
        ;   Stack status for callin TransmitConfirm
        ;
        ;       |             |
        ;       |  ProtDS     | 2
        ;       |  Status     | 2
        ;       |  ReqHandle  | 2
        ;       |  MACID      | 2
        ;       |  ProtID     | 2
        ;       +-------------+
        ;
        push    [si].TCS_ProtId         ;push arguments for Transmit_Confirm
        push    CommChar.cc_moduleid    ;MAC module ID
        push    ax                      ;request handle
        mov     ax,SUCCESS              ;figure out transmit result
        push    ax                      ;push transmit status
        push    ProtDS                  ;push protocol DS
        call    [TransmitConfirm]       ;call Transmit_Confirm
        POP     di
        jmp     @F
xd_done:
@@:     DWADD   MacStat.ms_TxTotPktCT,1 ;keep status

        call    Start_XMit

        jmp     again

xd_exit:


just_int:                                 ;enable interrupt

;inserted on 2006/02/16 to debug	begin
;	pushf
;	push	ax
;	mov	al,'x'
;	call	pchar
;	pop	ax
;	popf
;inserted on 2006/02/16 to debug	end


;       sti
        pop     di                      ;restore di, si, es, flags
        pop     si
        pop     es
        popf
        ret
A_ENDP  Xmit_Done

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;input: eax (it will dalay eax*30ns );;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
        public wait1_pci
wait1_pci:
        push    eax
        push    ebx
        push    ecx
        push    edx

        mov     dx,cs:IOBase
        add     dx,TCTR
        out     dx,eax
        mov     ebx,eax
wait_pci_1:
        in      eax,dx
        cmp     eax,ebx
        jbe     wait_pci_1     ;eax < ebx
        pop     edx
        pop     ecx
        pop     ebx
        pop     eax
        ret




COMMENT ^
----------------------------------------------------------------------------
$Name: TransmitChain - Accept a packet from the protocol $
----------------------------------------------------------------------------
Calling procedure:
        PUSH    WORD    ProtId          ;Module Id of protocol
        PUSH    WORD    ReqHandle       ;handle of this request
        PUSH    LPBUF   TxBufDescr      ;pointer to frame buffer descriptor
        PUSH    WORD    MacDS           ;DS of MAC module
        CALL    TransmitChain

$Description:
This procedure fields the transmit request from the upper layers. It gets
a Transmit Control Block if one is available. If not, we are out of resources.
If one is, we blow the data onto the board at the spot indicated by the
transmit control structure. We then call Start_Xmit to start the transmission
if possible.
$
^
A_PROC          TransmitChain, FAR

MacDS           EQU     <WORD PTR [bp+6]>    ;arguments
TxBufPT         EQU     <DWORD PTR [bp+8]>
TxBufSeg        EQU     <WORD PTR [bp+10d]>
ReqHandle       EQU     <WORD PTR [bp+12]>
ProtId          EQU     <WORD PTR [bp+14]>

CallDS          EQU     <WORD PTR [bp-2]>    ;local variables
PacketSize      EQU     <WORD PTR [bp-4]>


                push    bp              ;set up stack frame
                mov     bp,sp           ;set bp is equal sp
                sub     sp,6d           ;room for locals

                mov     CallDS,ds       ;save callers ds
                mov     ds,MacDS        ;and get ours

                push    si              ;save registers used si, di
                push    di

                CLI                     ;clear interrupt


;inserted on 2006/02/16 to debug	begin
;	pushf
;	push	ax
;	mov	al,'T'
;	call	pchar
;	pop	ax
;	popf
;inserted on 2006/02/16 to debug	end


                push    ax
                mov     al,ds:debug1
                test    al,1
                pop     ax
                jnz     @F
                call    ResetNIC

@@:
                mov     ds:debug1,1
                call    Xmit_Done

;we save all these things because our DS won't be available when we need them.
                mov     PacketSize,0    ;initial packet size

;Get a Tx_Control structure. If none is available, then there is an error.
;If one is available, fill in the necessary fields

 ;3/28/98       sti                     ;enable interrupt
                mov     di,TCSNextToFill;get next Tx_Control to fill
                ;cmp     TxCount,8
                ;jb      @F
                mov     al,TCS_USED     ;mark as used
                xchg    [di].TCS_Free,al
                cmp     al,TCS_AVAIL    ;was it free?
                je      @F              ;yes
                jmp     err             ;no. out of resources
@@:             mov     ax,[di].TCS_Link;update pointer to next Tx_control
                mov     TCSNextToFill,ax;refresh next TCS to fill

 ;3/28/98       sti                     ;enable interrupt
                mov     ax,ReqHandle    ;store request params
                mov     [di].TCS_Handle,ax
                mov     ax,ProtId       ;store protocol ID
                mov     [di].TCS_ProtId,ax
                push    di              ;save Tx Control Structure

;12/6/95
;Prepare to do Remote DMA in order to transfer datas from TxBufDescr
;(TxBufTP) to the local memory of adapter
;BX : store remote address
;CX : transmit counter
;SI : source address for remote write

  ;add check own 0-3 bit
                xor     bx,bx                   ;clear bx for remote address
                mov     bx,[di].TCS_PStart      ;get board buffer address
;check for immediate data
                les     di,TxBufPT              ;get address of descriptor
                mov     cx,es:[di].tx_immedlen  ;length of immediate data
                cmp     cx,0                    ;is there any?
                je      physaddrs               ;without immediate data

                lds     si, es:[di].tx_immedpt  ;address of immediate data


                add     PacketSize,cx           ;update packetsize
                cli
;copy immediate data to board. The board address will always be word aligned
;at this point.
                push    ax
                push    bx
                push    cx
                push    dx
                push    di
                push    es
                push    si
                push    ds
                mov     es,MacDS
                mov     di,bx

                shr     cx,1
                pushf
                shr     cx,1
                pushf
                rep     movsd                   ;mov es:di< ds:si
                popf
                jnc     d1
                movsw
           d1:
                popf
                jnc     d2
                movsb
           d2:
                pop     ds
                pop     si
                pop     es
                pop     di
                pop     dx
                pop     cx
                pop     bx
                pop     ax

                add     bx, cx                  ;update new memory address

physaddrs:

                mov     ax,es:[di].tx_blkct     ;get number of buffers
                cmp     ax,0                    ;any to copy?
                je      cpydone                 ;no. we're done
                lea     di,es:[di].tx_datablk   ;first data block decriptor

;if the data pointer is a physical address, it must be converted to a virtual
;address. Since we are always in protected mode, no mode change can occur
;in this call.
;
;We use the following registers in this loop:
;       cx:    length of data to xfer
;       bx:    offset into TxBufDescr of block to xfer
;       es:di  board location to xfer to

loop1:                                          ;save number of blocks
                push    ax
                mov     es, TxBufSeg            ;get ES

                mov     cx,es:[di].tblk_datalen ;length of buffer
                cmp     es:[di].tblk_ptrtype,PHYSTYPE;is the pointer physical?
                je      physconv
                lds     si, es:[di].tblk_datapt ;no. use virtual addr
                jmp     packinfo

physconv:


IFDEF           DOS
                lds     si, es:[di].tblk_datapt ;DOS always uses seg:off format
ELSE                                            ;OS/2
                ;************************ <OS/2> **************************
                CLI

                push    bx
                push    cx
                push    dx
                mov     ax,ES:[di].tblk_datapt._hi ;high order of phys addr
                mov     bx,ES:[di].tblk_datapt._lo ;low order of phys addr
                mov     ds,cs:cs_ds
                mov     si,segTX
                mov     dl,DEVHLP_PHYSTOGDTSEL  ;opcode
                call    ds:Device_Help
                mov     ds,si
                xor     si,si
                pop     dx
                pop     cx
                pop     bx
                ;************************ <OS/2> **************************
ENDIF

packinfo:
                add     PacketSize,cx           ;update PacketSize
                CMP     CX, 0                   ;is counter = 0 ?
                JZ      Zero_Len
                cli
                push     ax
                push     bx
                push     cx
                push     dx
                push     di
                push     es
                push     si
                push     ds
                mov      es,MacDS
                mov      di,bx

                shr     cx,1
                pushf
                shr     cx,1
                pushf
                rep     movsd                   ;mov es:di< ds:si
                popf
                jnc     d3
                movsw
           d3:
                popf
                jnc     d4
                movsb
           d4:

                pop     ds
                pop     si
                pop     es
                pop     di
                pop     dx
                pop     cx
                pop     bx
                pop     ax

Zero_Len:       add     bx, cx                  ;get new remote address

@@:             pop     ax                      ;retrieve # of blocks
                add     di, SIZE Tdatablk       ;increment index to TXBufDescr
                dec     ax
                jnz     loop1                   ;loop if more buffers to copy

cpydone:
                mov     ds,MacDS                ;go back to our dataseg
                mov     cx,PacketSize           ;get size of packet
                cmp     cx,MINPACKET            ;is it long enough?
                jae     @F                      ;yes
                mov     cx,MINPACKET            ;no. set to min. packet size

@@:             pop     di                      ;get back Tx_Control Structure
                mov     [di].TCS_Size,cx        ;set the size

                inc     TxCount
                call    Start_Xmit              ;start transmit if possible

                mov     ax,REQUEST_QUEUED
done:

;inserted on 2006/02/16 to debug	begin
;	pushf
;	push	ax
;	mov	al,'t'
;	call	pchar
;	pop	ax
;	popf
;inserted on 2006/02/16 to debug	end

                sti
                mov     ds,CallDS               ;restore caller's DS
                pop     di
                pop     si
                add     sp,6d
                pop     bp
                ret     10d

;deal with errors
err:

;inserted on 2006/02/16 to debug	begin
;	pushf
;	push	ax
;	mov	al,'S'
;	call	pchar
;	pop	ax
;	popf
;inserted on 2006/02/16 to debug	end

                mov     ax,OUT_OF_RESOURCE
                jmp     done

A_ENDP          TransmitChain

COMMENT ^
----------------------------------------------------------------------------
$Name:  MyTimerRoutine          - Handle Xmit Timeouts
----------------------------------------------------------------------------
;hold timeout retransmit or send next packet.

$Description:
This routine is run off of the timer thread.
This procedure handles the case of a timed out transmission. This happens
sometimes, and it is unclear whether it is a bug in the driver or the
DP8029. If we time out, we put the NIC into loopback and wait for the
transmit to end for a while. This is to handle the case of the Cheapernet
cable being removed, which would cause a xmit to never terminate. After
this, we call xmit_done to terminate the transmit.

We time out after about 550 ms (10 ticks). This is longer than the longest
possible transmit with collisions on Ethernet.
$
^
L_PROC          MyTimerRoutine FAR
IFDEF   DOS                   ;DOS

                push    ax    ;save registers
                push    bx
                push    cx
                push    dx
                push    si
                push    di
                push    ds
                push    es

                push    cs
                pop     ds    ;make ds is equal to cs

ELSE                          ;OS/2
                ;************************ <OS/2> **************************
                pusha
                push    es
                ;************************ <OS/2> **************************
ENDIF


                pushf                           ;reserve flags
                cli
                                        ;clear interrupt
                inc     TickCount               ;keep time
                cmp     XmitInProg,1            ;are we transmitting?
                jne     TimerDone               ;yes
                mov     ax,TickCount            ;get current time
                sub     ax,XmitStartTime
                cmp     ax,25                   ;about 750 ms
                jna     TimerDone               ;not timed out
                call    Xmit_Done               ;call xmit done


                call    ResetNIC


                DWADD   MacStat.ms_TxTimeoutPktCT,1 ;keep status
TimerDone:      popf

;v200 8/27 add by Jack Chen  (Reason for Nway ML6692 link failure)
;
                inc     Active_count
                cmp     Active_count,30         ;about 1 second
                jae     Check_Turbo_Link        ;check turbo link
                jmp     No_Polling

Check_Turbo_Link:

                mov     dx, (DS:NICIDR0)
                add     dx, 074h
                in      ax,dx
                test    ax,SIZE16_BIT10         ;test 74h<10>=1 exit
                jnz     no_link_v200_1          ;=1 exit
                                                ;=0
                mov     dx,(DS:NICIDR0)
                add     dx,070h
                in      ax,dx
                test    ax,SIZE16_BIT0          ;test 70h<0>=0 exit
                jz      no_link_v200_1          ;=0 exit
                                                ;=1
                mov     dx,(DS:NICIDR0)
                add     dx,074h
                mov     ax,0f3c0h
                out     dx,ax

no_link_v200_1:
                mov     Active_count,0
No_Polling:

IFDEF           DOS                             ;DOS
                pop     es                      ;Pop all registers
                pop     ds
                pop     di
                pop     si
                pop     dx
                pop     cx
                pop     bx
                pop     ax
                jmp     cs:NextTimer            ;original Timer vector
ELSE                                            ;OS/2
                ;************************ <OS/2> **************************
                pop     es
                popa
                retf
                ;************************ <OS/2> **************************
ENDIF
A_ENDP          MyTimerRoutine

CODESEG_END

INITCODESEG_BEG

COMMENT ^
----------------------------------------------------------------------------
$Name: Init_Transmit - Initialize the Transmit module
----------------------------------------------------------------------------
$Description:
This module initializes the transmit data structures and the timer interrupt.
$

$Method:
$
^

A_PROC      Init_Transmit

;set up timer interrupt
IFDEF   DOS                                     ;DOS
                push    es
                mov     ah,35h                  ;get current int vector
                mov     al,1Ch                  ;timer interrupt
                int     21h                     ;get the vector
                mov     NextTimer._SEG,es       ;and save it
                mov     NextTimer._OFF,bx

                mov     ah,25h                  ;set int. vector
                mov     dx, OFFSET DGROUP:MyTimerRoutine   ;addr of routine
                mov     al,1Ch                  ;timer interrupt
                int     21h
                pop     es
ELSE                                            ;OS/2
                ;************************ <OS/2> **************************
                mov     ax, OFFSET CGROUP:MyTimerRoutine   ;addr of routine
                mov     dl,DEVHLP_SETTIMER              ;get timer interrupt
                call    [Device_Help]
                ;************************ <OS/2> **************************
ENDIF

                mov     ax,SUCCESS
                ret
A_ENDP  Init_Transmit

COMMENT ^
----------------------------------------------------------------------------
$Name: RestoreTimer - restore the timer interrupt
----------------------------------------------------------------------------
$Description:
This procedure is called if initialization fails after we have obtained
the timer interrupt. It removes us from the timer chain.
$

$Method:
$
^
A_PROC          RestoreTimer
IFDEF DOS
                push    ds
                mov     ah,25h                  ;set int. vector
                mov     dx, NextTimer._OFF      ;addr of routine
                mov     ds, NextTimer._SEG
                mov     al,1Ch                  ;timer interrupt
                int     21h
                pop     ds
ELSE                                            ;OS/2
                ;************************ <OS/2> **************************
                mov     ax, OFFSET MyTimerRoutine
                mov     dl,DEVHLP_RESETTIMER
                call    [Device_Help]
                ;************************ <OS/2> **************************
ENDIF
                ret
A_ENDP          RestoreTimer

INITCODESEG_END
END
